;;;;;;;;;;;;
;;
;; Libraries
;;
;;;;;;;;;;;;
(ns europe-pnas.core
  (:gen-class)
  (:require [clojure.set :as set])
  (:require [quil.core :as q])
  (:use [incanter core io charts stats pdf])
  (:import java.awt.Font)
  (:import java.awt.Shape))

(defn -main [])
BREAK ;; To stop LightTable from evaluating the whole file when I connect it to the JVM.


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; Extending Incanter's Drawing functions
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defn last-series [chart] (dec (.getDatasetCount (.getPlot chart))))

(defn remove-from-legend [plot n]
;;  http://stackoverflow.com/questions/3342406/jfreechart-how-to-hide-items-from-legend
  "Removes the legend item n from a plot."
  (let [oldlegend (.getLegend plot)
	olditems (.getLegendItems (.getPlot plot))
	c (.getItemCount olditems)
	newitems (new org.jfree.chart.LegendItemCollection)]
    (dorun (for [r (range 0 c) :when (not= r n)]
	     (.add newitems (.get olditems r))))
    (.setFixedLegendItems (.getPlot plot) newitems)))

(def color-list
  "from http://stackoverflow.com/questions/309149/generate-distinctly-different-rgb-colors-in-graphs"
  (let [raw-list ["000000"
"00FF00" "0000FF" "FF0000" "01FFFE" "FFA6FE" "FFDB66" "006401" "010067" "95003A" "007DB5" "FF00F6" #_"FFEEE8" "774D00" "90FB92" "0076FF" "D5FF00" "FF937E" "6A826C" "FF029D" "FE8900" "7A4782"
"7E2DD2" "85A900" "FF0056" "A42400" "00AE7E" "683D3B" "BDC6FF" "263400" "BDD393" "00B917" "9E008E" "001544" "C28C9F" "FF74A3" "01D0FF" "004754" "E56FFE" "788231" "0E4CA1" "91D0CB" "BE9970"
"968AE8" "BB8800" "43002C" "DEFF74" "00FFC6" "FFE502" "620E00" "008F9C" "98FF52" "7544B1" "B500FF" "00FF78" "FF6E41" "005F39" "6B6882" "5FAD4E" "A75740" "A5FFD2" "FFB167" "009BFF" "E85EBE"]]
    (vec (map (fn [hexcode] (let [[r g b] (map read-string (map #(str "0x" (apply str  %)) (partition 2 hexcode)))] (new java.awt.Color r g b))) raw-list))))

(defn set-color [plot & offset]
  (let [data-number (+ (last-series plot) (if offset offset 0))]
    (set-stroke-color plot (color-list data-number))))

(defn theme-plot [plot]
  (.setAxisLineVisible (.getRangeAxis (.getPlot plot)) true)
  (.setLabelPaint (.getRangeAxis (.getPlot plot)) java.awt.Color/black)
  (.setTickLabelPaint (.getRangeAxis (.getPlot plot)) java.awt.Color/black)
  (.setTickMarkPaint (.getRangeAxis (.getPlot plot)) java.awt.Color/black)
  (.setAxisLinePaint (.getRangeAxis (.getPlot plot)) java.awt.Color/black)
  (.setLabelFont (.getRangeAxis (.getPlot plot)) (new Font "Arial" Font/PLAIN 16))
  (.setTickLabelFont (.getRangeAxis (.getPlot plot)) (new Font "Arial" Font/PLAIN 12))
  (.setAxisLineVisible (.getDomainAxis (.getPlot plot)) true)
  (.setLabelPaint (.getDomainAxis (.getPlot plot)) java.awt.Color/black)
  (.setTickLabelPaint (.getDomainAxis (.getPlot plot)) java.awt.Color/black)
  (.setTickMarkPaint (.getDomainAxis (.getPlot plot)) java.awt.Color/black)
  (.setAxisLinePaint (.getDomainAxis (.getPlot plot)) java.awt.Color/black)
  (.setLabelFont (.getDomainAxis (.getPlot plot)) (new Font "Arial" Font/PLAIN 16))
  (.setTickLabelFont (.getDomainAxis (.getPlot plot)) (new Font "Arial" Font/PLAIN 12))
  (.setBackgroundAlpha (.getPlot plot) 0.0)
  (.setRangeGridlinesVisible (.getPlot plot) false)
  (.setRangeGridlinePaint (.getPlot plot) java.awt.Color/lightGray)
  (.setAxisLineVisible (.getRangeAxis (.getPlot plot)) true)
)


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; SHAPEFILE AND MAP PROJECTION FUNCTIONS
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defn process-shapes [line]
  (let [[shapetype & rawdata] (clojure.string/split line #" ")
        rawdata (read-string (first (clojure.string/split (apply str (interpose " " rawdata)) #"\"")))]
    (case shapetype
      "\"LINESTRING" (hash-map :polygons (list rawdata)) ;; silkroad, gridlines, countrylines
      "\"MULTILINESTRING"  (hash-map :polygons (apply concat rawdata))
      "\"POLYGON" (hash-map :polygons rawdata) ;; lakes, landshapes, sylvaticplague
      "\"MULTIPOLYGON" (hash-map :polygons (apply concat rawdata))))) ;; landshapes

(declare latlon-to-xy)

(defn plot-shapes [shape & opts]
  "Plots shapes. The partition 2 or partition 3 bit is because the
   silkroad kml file for some reason has LONGS interposed between the
   coordinates. Also prevents plotting of the whole earth."
  (let [options (when opts (apply assoc  {} opts))]
    (doseq [polygons shape]
      (doseq [polygon (:polygons polygons)]
        (let [p (filter #(and (< (get options :c1 -10) (second %) (get options :c2 85)) (< (get options :c3 -40) (first %) (get options :c4 140))) (if (instance? Long (last (take 3 polygon))) (map butlast (partition 3 polygon)) (partition 2 polygon)))]
          (when (seq p)
            (q/begin-shape)
            (doseq [v p]
              (apply q/vertex (latlon-to-xy (reverse v)))) ;; This reverse is because my shapes are in lon/lat
            (if (get options :close) (q/end-shape :close) (q/end-shape))))))))

(defn globe-distance
  "Calculates distance over the globe, given lat-lon. Uses Haversine Algorithm: http://www.movable-type.co.uk/scripts/latlong.html"
  [[lat1 lon1] [lat2 lon2]]
  (let [r 6371
        dlat2 (/ (q/radians (- lat2 lat1)) 2)
        dlon2 (/ (q/radians (- lon2 lon1)) 2)
        rlat1 (q/radians lat1)
        rlat2 (q/radians lat2)
        a (+ (* (sin dlat2) (sin dlat2)) (* (sin dlon2) (sin dlon2) (cos rlat1) (cos rlat2)))
        c (* 2 (atan2 (sqrt a) (sqrt (max 0 (- 1 a)))))] ;; the max is here for cases where a is by rounding errors slightly above 1. (1.0000000010681909), as is the case for (globe-distance [-8.83 42.57] [171.1522085336539 -42.545628004807696])
    (* r c)))

(defn setup-europe []
  (q/smooth)
  (q/frame-rate 60)
  (q/text-font (q/create-font "Arial Bold" 28 true))
  (q/set-state! :year-counter (atom 1340) :cutoff-vector (atom [0.0001 0.0002 0.0005 0.001 0.002 0.005 0.01 0.02 0.05 0.1 0.2 0.3 0.4 0.5 0.7 0.95 1.0]) :cutoff (atom 0)
              :delay (atom 0) :delay-vector (atom [0 1 2 3 4 5 6 7 8 9])
              :indicator (atom 2) :indicator-vector (atom [:z-score-wet-central1 :z-score-wet-central2 :z-score-wet-germany :z-score-temp-merveille :z-score-temp-alps1 :z-score-temp-alps2 :z-score-temp-alps3 :z-score-temp-spain :z-score-wettemp-albania :temp-rcs :temp-ltmrcs :temp-holocene :precip-sheppard :precip-kara :drought-kazakh :plmean :pl5 :pl95])
              :dataset (atom 0) :dataset-vector (atom [`win1-min3-max6-gap2 `win1-min3-max6-gap2 `win1-min4-max7-gap2 `win2-min4-max6-gap2]))
  (q/background 255))

;; Globe Projections
;;;;;;;;;;;;;;;;;;;;
(defn prepare-ortho []
  (def zoom (atom 0.35)) ;; 0.75
  (def x-offset (atom 330))
  (def y-offset (atom 350))
  (def mouse-pos (atom [0 0]))
  (defn latlon-to-xy
    "Converts lat/lon to radians, and calculates X,Y on a Orthographic map"
    [[lat lon]]
    (let [radius 6371
          clon (q/radians 20)
          clat (q/radians 40) ;; Caffa
          xoffset (+ (* @zoom 4.125) @x-offset)
          yoffset (+ (* @zoom 21.0) @y-offset)
          xblowup (* @zoom 0.18125 2)
          yblowup (* @zoom 0.1825 2)
          round-ten (fn [n] (/ (q/round (* 10 n)) 10.0))]
      (vector (+ xoffset (* xblowup radius (cos (q/radians lat)) (sin (- (q/radians lon) clon))))
              (- yoffset (* yblowup radius (- (* (cos clat) (sin (q/radians lat)))
                                              (* (sin clat) (cos (q/radians lat)) (cos (- (q/radians lon) clon))))))))))

(defn prepare-kavraiskiy-europe []
  (def zoom (atom 8))
  (def x-offset (atom -210))
  (def y-offset (atom -240))
  (def mouse-pos (atom [0 0]))
  (defn latlon-to-xy
    "Converts lat/lon to radians, and calculates X,Y on a Kavraiskiy map"
    [[lat lon]]
    (let [xoffset (+ (* @zoom 41.25) @x-offset) yoffset (+ (* @zoom 210) @y-offset) xblowup (* @zoom 181.25) yblowup (* @zoom 182.5)
          round-ten (fn [n] (/ (q/round (* 10 n)) 10.0))]
      (vector (round-ten (+ xoffset (* xblowup 3/2 (q/radians lon) (/ q/PI) (sqrt (- (/ (sq q/PI) 3.0) (sq (q/radians lat))))))) (round-ten (- yoffset (* yblowup (q/radians lat))))))))

(defn prepare-kavraiskiy-europe2 []
  (def zoom (atom 6))
  (def x-offset (atom -100))
  (def y-offset (atom -140))
  (def mouse-pos (atom [0 0]))
  (defn latlon-to-xy
    "Converts lat/lon to radians, and calculates X,Y on a Kavraiskiy map"
    [[lat lon]]
    (let [xoffset (+ (* @zoom 41.25) @x-offset) yoffset (+ (* @zoom 210) @y-offset) xblowup (* @zoom 181.25) yblowup (* @zoom 182.5)
          round-ten (fn [n] (/ (q/round (* 10 n)) 10.0))]
      (vector (round-ten (+ xoffset (* xblowup 3/2 (q/radians lon) (/ q/PI) (sqrt (- (/ (sq q/PI) 3.0) (sq (q/radians lat))))))) (round-ten (- yoffset (* yblowup (q/radians lat))))))))

(defn prepare-carre-world []
  (def zoom (atom 5.5))
  (def x-offset (atom 70))
  (def y-offset (atom 340))
  (def mouse-pos (atom [0 0]))
  (defn latlon-to-xy
    "Converts lat/lon to radians, and calculates X,Y on Carre Plate map"
    [[lat lon]]
    (let [round-ten (fn [n] (/ (q/round (* 10 n)) 10.0))]
      (vector (round-ten (+ @x-offset (* @zoom lon))) (round-ten (- @y-offset (* @zoom lat)))))))


;;;;;;;;;;;;;;;;;;;
;;
;; VARIOUS DATASETS
;;
;;;;;;;;;;;;;;;;;;;

;; Shapesfiles from naturalearthdata.com
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(def landshapes (map process-shapes (rest (clojure.string/split (slurp "resources/geographic/ne_50m_land.csv") #"\n"))))
(def caspian-sea (list (hash-map :polygons (list (second (:polygons (first (take-last 40 landshapes))))))))
(def gridlines (map process-shapes (rest (clojure.string/split (slurp "resources/geographic/ne_10m_graticules_10.csv") #"\n"))))
(def lakes (map process-shapes (rest (clojure.string/split (slurp "resources/geographic/ne_50m_lakes.csv") #"\n"))))
(def sylvaticplague (map process-shapes (rest (clojure.string/split (slurp "resources/geographic/Easterday-sylvatic-plague-foci.csv") #"\n"))))
(def silkroad (map process-shapes (rest (clojure.string/split (slurp "resources/geographic/silkroad-all-but-seas.csv") #"\n"))))
(def searoutes (map process-shapes (rest (clojure.string/split (slurp "resources/geographic/combined-maritime-traderoutes.csv") #"\n"))))
(def tearoad (map process-shapes (rest (clojure.string/split (slurp "resources/geographic/tea-route-reconstructed.csv") #"\n"))))


;; Tree-ring data
;;;;;;;;;;;;;;;;;
(def climate-db-raw (read-string (slurp "resources/tree-rings-processed.txt")))

(def climate-info
  (hash-map :z-score-wet-central1  {:center [49 12.5] :name "Central Europe 1" :source "Buentgen et al, QSR 2011"}
            :z-score-wet-central2  {:center [51 10] :name "Central Europe 2" :source "Buentgen et al, Science 2011"}
            :z-score-wet-germany   {:center [51.5 9.5] :name "Northern Germany" :source "Buentgen et al, QSR 2010"}
            :z-score-temp-merveille {:center [44.1 7.5] :name "Maritime French Alps" :source "Buentgen et al Climate Change 2012" :sites [[44.1 7.4666667] [44.083332 7.4666667] [44.05 7.483333] [44.05 7.45]]}
            :z-score-temp-alps1 {:center [46.8 10.5] :name "Alpine Arc" :source "Buentgen et al, Trace 2009"}
            :z-score-temp-alps2 {:center [46.3 9.7] :name "Swiss and Western Austrian Alps" :source "Buentgen et al, Climate Dynamics 2005" :sites [[46.433334 7.8166666] [46.433334 8.183333] [46.2 8.066667] [46.466667 9.783334] [46.0 11.5]]}
            :z-score-temp-alps3 {:center [46 10] :name "Italian Alps" :source "see Manuscript"}
            :z-score-temp-spain {:center [42.5 0.6] :name "Pyrenees" :source "Buentgen et al, Trees 2010" :sites [[42.6 1.8] [42.4 0.7] [42.44 0.47] [42.59 0.45] [42.58 0.44] [42.48 0.44] [42.46 0.37] [42.41 0.34] [42.3 0.13] [42.42 -0.06] [42.51 -0.09] [42.48 -0.42] [42.5 -0.47] [42.38 -0.59] [42.97 -0.7] [42.98 -0.71] [43.0 -0.74] [42.99 -0.75] [43.0 -1.06] [43.0 -1.06] [42.32 -1.25] [42.32 -1.3] [42.36 -2.03] [42.36 -2.04] [42.28 -2.07] [42.27 -2.23] [42.28 -2.24] [42.29 -2.25]]}
            :z-score-wettemp-albania {:center [41.6 20.4] :name "Balkan Area" :source "Seim et al, Climate Research 2012" :sites [[42.416668 19.766666] [41.8 20.233334] [40.7 21.333334]]}
            :precip-sheppard {:center [36.5 98.3] :name "Northeast Qinghai" :source "Sheppard et al, Climate Dynamics 2004" :sites [[37.0 98.5] [36.0 98.0]]}
            :precip-kara {:center [36.6 75.1] :name "Karakorum 1 (d18-O)":source "Treydte et al, Nature 2006"  :sites [[35.366665 74.86667] [36.033333 74.583336] [36.033333 74.583336] [36.583332 75.083336]] :notes "Only the last location was used for <1900 years"}
            :temp-rcs {:center [37.8 73.9] :name "Tien Shan & Karakorum" :source "Esper et al, Trees 2007" :sites [[40.166668 72.583336] [40.166668 72.583336] [40.166668 72.583336] [39.833332 71.5] [40.2 72.61667] [39.833332 71.5] [40.2 72.61667] [35.166668 75.5] [35.166668 75.5] [36.583332 75.083336] [36.583332 75.083336] [36.583332 75.083336] [36.583332 75.083336] [36.333332 74.03333] [36.333332 74.03333] [36.033333 74.583336] [36.233334 74.98333]] :notes "only the high sites, excluding Hun"}
            :wettemp-holocene {:center [36 74] :name "Karakorum 2 (TRW)" :source "Esper et al, The Holocene 2002" :notes "site location given as a square"}
            :temp-ltmrcs {:center [40.1 72.2] :name "Tien Shan" :source "Esper et al, Climate Dynamics 2003" :sites [[39.833332 71.5 ] [40.2 72.61667] [40.166668 72.583336]]}
            :precip-mongol {:center [48.3 99.4] :name "Central Mongolia":source "Pederson et al, PNAS 2014"  :sites [[48.3 99.4]] :notes "Site location estimated from figure in paper"}
            ))

(def i-europe (into (sorted-set) [:z-score-wet-central1 :z-score-wet-central2 :z-score-wet-germany :z-score-temp-merveille :z-score-temp-alps1 :z-score-temp-alps2 :z-score-temp-alps3 :z-score-temp-spain :z-score-wettemp-albania]))
(def i-asia (into (sorted-set) [:precip-sheppard :precip-kara :temp-rcs :temp-ltmrcs :wettemp-holocene :precip-mongol]))


;; Gridded PDSI data Asia (Cook)
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(def pdsi-asia (into (sorted-map) (apply merge (map #(hash-map (int (:year %)) (dissoc % :year)) (read-string (slurp "resources/pdsi-asia-unprocessed.txt"))))))


;; Plague outbreak dataset
;;;;;;;;;;;;;;;;;;;;;;;;;;
(def plague-db (read-string (slurp "resources/plague-db-europe.txt")))

(def plague-db-rollupyears
  (let [unsorted (apply merge-with concat (map #(hash-map (:name %) (vector (:year %))) plague-db))]
     (zipmap (keys unsorted) (map #(vec (sort %)) (vals unsorted)))))

(def city-to-latlon (zipmap (map :name plague-db) (map #(vector (:lat %) (:lon %)) plague-db)))

;; Dols dataset on Asian plague. Only using southern russia and caucasus
;; assuming that plague after the BD took the same route as the BD
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(def asian-plague (apply merge (read-string (slurp "resources/dols-middle-east.txt"))))

;; Harbors of Medieval Europe:
;;
;; Three sources:
;;
;; Chief trade routes in Europe, Levant and North Africa 1300-1500 CE.
;; CIOLEK: http://www.ciolek.com/owtrad/data/tmcZWEm1300a.html
;;
;; An untitled map of (apply merge 'the chief trade-routes of the Middle Ages.' The map is published on pp. 224-225,)
;; King, Donald. 1985. Currents of Trade: Industries, Merchants and Money. pp. 200-230, in Evans, Joan
;; (ed.). 1985. The Flowering of the Middle Ages. London: Thames and Hudson Ltd.
;;
(def mainseatrade-data (set (filter #(= (:type %) "sl")
                                    (map #(let [[n1 _ n2 _ _ _ type _ _ _ _ _ _ _ _ _ _ _ _ lon1 lat1 _ lon2 lat2] (clojure.string/split % #",")]
                                            (hash-map :n1 n1 :n2 n2 :type type :coord1 [(Float/parseFloat lat1) (Float/parseFloat lon1)] :coord2 [(Float/parseFloat lat2) (Float/parseFloat lon2)]))
                                         (rest (clojure.string/split-lines (slurp "resources/major-routes-europe.owtrad")))))))

(def harbors-europe-evans
  ;; A good rule to find those harbors in Evans that are also in the plague db seems to be to stipulate that they are within 10km of each other.
  (let [lookup (apply merge (map #(hash-map (:name %) (vector (:lat %) (:lon %))) plague-db))]
    (set (map #(first (last %)) (remove nil? (for [coord (mapcat #(vector (:coord1 %) (:coord2 %)) mainseatrade-data)]
                                          (let [answer (take-while #(< (globe-distance (lookup %) coord) 10) (sort-by (fn [city] (globe-distance (lookup city) coord)) (set (map :name plague-db))))]
                                            (when (seq answer)
                                              (vector coord (int (globe-distance (lookup (first answer)) coord)) (int (globe-distance (lookup (last answer)) coord)) answer)))))))))

;;
;; Venetian Galley-operated trade routes 1400-1530CE.
;; http://www.ciolek.com/owtrad/data/tmcXMEm1400.html
;;
;; The map is published on pp. 398, together with notes notes on pp. 398
;; and 404 in: Spufford, Peter. 2002. Power and Profit: The Merchant in
;; Medieval Europe. London: Thames & Hudson.
;;
(def translate-venezia-harbors
  "translates the Ciolek Venetian network to the
  places in our digitized plague-db, when present."
  {"Aigues Mortes" "Montpellier" ;; former is the harbor area near the latter
  "Alexandria" "Alexandrie", "Algiers" "Alger", "Almeria" "Almería", "Barcelona" "Barcelona", "Beirut" "Beyrouth", "Bone" "Bone", "Bougie" nil ;; Bejaia, Bgayet, southern mediterranean
  "Bruges" "Brugge", "Cadiz" "Cádiz", "Candia" nil ;; Heraklion on Crete
  "Civitavecchia" "Civitavecchia", "Constantinople" "Constantinople", "Corfu" nil ;; Greece
  "Coron" nil ;; Koroni, Greece
  "Famagusta" nil ;; Ammochostos, Cyprus
  "Jerba" "Djerba", "Lisbon" "Lisboa", "Pula?" nil ;; Pula, Croatia
  "Dubrovnik?" "Dubrovnik" ;; Ragusa
  "London" "London", "Malaga" "Málaga", "Marseilles" "Marseille", "Melilla" nil ;; Morocco
  "Messina" "Messina", "Modon" nil ;; Methoni, Greece
  "Naples" "Napoli", "Negroponte" nil ;; Khalkis, Greece
  "Oran" "Oran", "Palermo" "Palermo", "Palma" "Palma", "Pisa" "Pisa", "Rhodes" nil ;; Rhodes Island, Greece
  "Sandwich" "Sandwich", ;; Bunch of sealocs, not included.
  "Sinope" nil ;; Sinop, Black Sea, northern Turkey
  "Southampton" "Southampton", "Syracuse" "Siracusa", "Tana" "Rostov (Azov)", "Trebizond" nil ;; Trabzon, Black Sea, Turkey.
  "Tripoli" "Tripoli", "Tunis" "Tunis", "Valencia" "Valencia", "Venice" "Venezia", "Zara" "Zadar"})


;; Check which harbors are not found manually in the above translate-variable, but might be in the dataset, by looking
;; at entries in Ciolek that are not matched to plague-db, but within 50km of another entry in plague-db
;;
(let [veneziantrade-data (set (map #(let [[n1 _ n2 _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ lon1 lat1 _ lon2 lat2] (clojure.string/split % #",")]
                           (hash-map :n1 n1 :n2 n2 :coord1 [(Float/parseFloat lat1) (Float/parseFloat lon1)] :coord2 [(Float/parseFloat lat2) (Float/parseFloat lon2)]))
                           (rest (clojure.string/split-lines (slurp "resources/venezian-network.owtrad")))))
      harbor-cities-matched (set (concat (keys (select-keys translate-venezia-harbors (keys (filter (fn [[k v]] (seq v)) translate-venezia-harbors))))
                                 harbors-europe-evans))
      venezian-unmatched (remove #(or (harbor-cities-matched (:n1 %)) (harbor-cities-matched (:n2 %))) veneziantrade-data)
      ;; Merged the coordinates provided by Ciolek to those in our plague-db.
      ;; Doing so by getting the right name, and stipulating that they be within 50km of each other.
      harbors-venezia-spufford (let [lookup (apply merge (map #(hash-map (:name %) (vector (:lat %) (:lon %))) plague-db))]
                                 (remove nil?
                                  (for [[location coord] (set (mapcat #(vector (vector (:n1 %) (:coord1 %)) (vector (:n2 %) (:coord2 %))) venezian-unmatched))]
                                      (let [answer (remove #(harbor-cities-matched %) (take-while #(< (globe-distance (lookup %) coord) 50) (sort-by (fn [city] (globe-distance (lookup city) coord)) (set (map :name plague-db)))))]
                                         (when (seq answer)
                                            (vector location coord (int (globe-distance (lookup (first answer)) coord)) (int (globe-distance (lookup (last answer)) coord)) answer))))))]
  harbors-venezia-spufford)
;; Manually checking whether the following places could refer to the harbors listed after them, using Google Maps.
;;
;; ["sealoc04-tmc051112" [38.9 -9.9] 47 47 ("Cascais")]  ;; NO: Rather far out of the coast of Cascais. Very close to Lisbon as well
;; ["sealoc02-tmc051112" [35.95 -5.53] 20 31 ("Ceuta" "Gibraltar" "Tanger")] ;; NO: middle of the Strait of Gibraltar.
;; ["sealoc02-tmc050503" [43.3766 15.9879] 39 43 ("Šibenik" "Split" "Hvar")] ;; This sealoc is just outside these three harbors. They make sense as being part of the tradenetwork, but as they are not explicitly named in the Venezian dataset, or in Evans we leave them out for now.
;; ["loc02-tmc051112" [42.650555 18.088612] 38 38 ("Ston")] ;; This loc is Dubrovnik, which is already included in the dataset. Ston is nearby.
;;
;; No real reason to add one of these leftover points in Venezian Galley-operated traderoutes to the set of Medieval trading harbors.

;; Finally,
;;
;; Izmir added, as it is important, especially in the early modern period (and wasn't included in the trade route maps of the middle ages).
;; "The ottoman port of Izmir in the eighteenth and early nineteenth centuries, 1695 -1820", By Elena Frangakis.
;; &
;; Dols MW (1979) The second plague pandemic and its recurrences in the Middle East: 1347-1894. J Econ Soc Hist Orient 22(2):162–189.
;;
(def harbors (set (concat ["Izmir"] (remove nil? (set/union (set (vals translate-venezia-harbors)) harbors-europe-evans)))))
;;
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;





;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; SUPPLEMENTARY FIGURE 1 - DATASETS IN A CHART
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(prepare-ortho)
(reset! zoom 0.75)
(def x-offset (atom 700))
(def y-offset (atom 870))
;; 2100 pixels = 17.8cm * 300 DPI.
(q/defsketch europe-map :setup setup-europe :size [2100 1300]
  :renderer :pdf
  :output-file "results/FIGURE-S1-overview-datasets.pdf"
  :draw (fn []
          (q/background 150 185 230) ;; Ocean
          (q/stroke 250 240 220) (q/fill 250 240 220) (plot-shapes landshapes :c1 -10 :c2 85 :c3 -40 :c4 140)
          (q/stroke-weight 2)
          (q/stroke 110 140 200) (q/fill 0 0 0 0) (plot-shapes searoutes)
          (q/stroke-weight 1)
          (q/stroke 0 0 0 0) (q/fill 150 185 230) (plot-shapes caspian-sea)
          (q/stroke 0 0 0 0) (q/fill 150 185 230) (plot-shapes lakes)
          (q/stroke 250 200 70) (q/fill 250 200 70) (plot-shapes sylvaticplague)
          (q/stroke-weight 2)
          (q/stroke 130 130 70) (q/fill 0 0 0 0) (plot-shapes silkroad)
          (q/stroke 130 130 70) (q/fill 0 0 0 0) (plot-shapes tearoad)
          (q/stroke-weight 1)
          (q/stroke 100 100 100 150) (q/fill 0 0 0 0) (plot-shapes gridlines :c1 -10 :c2 80 :c3 -40 :c4 130)
          ;; Plague
          (doseq [[x y n] (map #(conj (latlon-to-xy (city-to-latlon %)) %) (sort-by #(count (plague-db-rollupyears %)) (keys plague-db-rollupyears)))]
            (let [c (count (plague-db-rollupyears n))]
              (if false #_(harbors n) (do (q/stroke 0 0 0) (q/fill 0 0 0)) (do (q/stroke 120) (q/fill 190)))
              (q/ellipse x y (+ 2 (sqrt (/ (* 14 c) q/PI))) (+ 2 (sqrt (/ (* 14 c) q/PI))))))
          ;; Tree-rings
          (doseq [regions (vals climate-info)]
              (let [[x y] (latlon-to-xy (:center regions))
                    w 40
                    h 40]
                (q/stroke 100 200 100 250) (q/fill 100 200 100 150)
                (q/ellipse x y w h)))
          ;; Close the sketch
          (q/exit)))
;; Batch Converts to fix ppi with these two commands.
;; mogrify -format png *.pdf
;; mogrify -units PixelsPerInch -density 300 *.png






;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; Functions to determine potential reintroductions from reservoirs
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defn mingap-years [years mingap]
  "Removes outbreak datapoints which occur too recently after a
  previous outbreak."
  (vec (concat (take 1 (sort years)) (map second (filter (fn [[y1 y2]] (>= (- y2 y1) (inc mingap))) (map vector (sort years) (rest (sort years))))))))

(defn local-cities [center range]
  "returns a set of cities near a centerpoint"
  (let [cities (set (map :name plague-db))]
    (set (filter (fn [city] (< (globe-distance center (city-to-latlon city)) range)) cities))))

(defn return-potential-outbreaks [center rangesize buffersize]
  "Finds new outbreaks in the center region (pooling cities there),
   which are not preceded by outbreaks in a larger region"
  ;; Cities in the center
  (let [sentinels (local-cities center rangesize)
        new-outbreaks (mingap-years (sort (set (mapcat plague-db-rollupyears sentinels))) 2)
        larger-region (local-cities center buffersize)
        ;; Outbreaks (and the year after in the larger region that do not overlap with the new outbreaks within the center.
        buffer-outbreaks-extended (set (mapcat #(range % (+ % 2)) (set/difference (set (mapcat plague-db-rollupyears larger-region)) new-outbreaks)))
        ;; the new outbreaks that are left are linked back to any sentinel in the region.
        sentinels-outbreaks (zipmap sentinels (map #(set/intersection (set (plague-db-rollupyears %)) (set/difference (set new-outbreaks) buffer-outbreaks-extended)) sentinels))]
    (select-keys sentinels-outbreaks (map first (remove (fn [[k v]] (nil? (seq v))) sentinels-outbreaks)))))



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; Generate some of the usage numbers of the dataset for the paper
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;; 46 Harbors in total.
(count harbors)

;; Total entries in plague-db
(count plague-db)

;; After BD
(count (filter #(> (:year %) 1354) plague-db))

;; fraction post-BD
(/ 7412 7711.0)

;; How many near a climate proxy?
(count (remove #(< % 1354) (mapcat plague-db-rollupyears (set (mapcat #(local-cities (:center (climate-info %)) 500) i-europe)))))

;; Fraction of the post-BD outbreaks near a climate proxy
(/ 4119 7711.0)

;; Calculate speed of plague in Europe, per year, using George Christakos' monthly speed.
;; Keep this in mind when setting the outter circle.
;; About 1000-1500km/year
(sort (map #(int (apply + %)) (partition 12 1 (map #(* 30 (read-string (second (clojure.string/split % #",")))) (rest (rest (clojure.string/split (slurp "resources/christakos-plague-speed-BD-permonth.csv") #"\n")))
     ))))
;;
;;
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;




;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; Explore the number of outbreaks included in the inner circle (default 500km), as we expand
;; it in size. The outter circle is set to be the same size as the inner circle.
;;
;; These figures are no longer included in the supporting information paper itself, as we were running out of supporting information slots (max 10).
;;
(def inner-sensitivity
  (for [info (vals (select-keys climate-info (filter i-europe (keys climate-info))))]
     (for [inner (range 50 1050 50)]
        (vector inner
                (:name info)
                (count (mingap-years (sort (set (mapcat plague-db-rollupyears (local-cities (:center info) inner)))) 2))))))

;; Clarifying plot!
(let [plot (scatter-plot [] [] :legend true)]
  (doseq [coll inner-sensitivity]
    (add-points plot (map first coll) (map last coll) :series-label (second (first coll)))
    (.setSeriesShape (.getRenderer (.getPlot plot) (last-series plot)) 0 (new java.awt.geom.Rectangle2D$Double -2.0, -2.0, 4.0, 4.0)))
  (set-theme-bw plot)
  (set-x-label plot "Inner circle radius (km)")
  (set-y-label plot "# of outbreak years")
  (remove-from-legend plot 0)
  (save-pdf plot "results/FIGURE-innerradius.pdf")
  (view plot))

(def outter-sensitivity
  (for [info (vals (select-keys climate-info (filter i-europe (keys climate-info))))]
     (for [outter (range 100 2100 100)]
        (vector outter
                (:name info)
                (count (set (mapcat #(range % (+ % 2)) (set (mapcat plague-db-rollupyears (local-cities (:center info) outter))))))
                ))))

(let [plot (scatter-plot [] [] :legend true)
      total-years (count (set (mapcat #(range % (+ % 2)) (set (flatten (vals plague-db-rollupyears))))))]
  (doseq [coll outter-sensitivity]
    (add-points plot (map first coll) (map #(/ (last %) total-years 0.01) coll) :series-label (second (first coll)))
    (.setSeriesShape (.getRenderer (.getPlot plot) (last-series plot)) 0 (new java.awt.geom.Rectangle2D$Double -2.0, -2.0, 4.0, 4.0)))
  (set-theme-bw plot)
  (set-x-label plot "Outer circle radius (km)")
  (set-y-label plot "% years excluded")
  (remove-from-legend plot 0)
  (save-pdf plot "results/FIGURE-outerradius.pdf")
  (view plot))
;;
;;
;; Basically, the only sensible alternative for 500 - 1000 (500km buffer around the inner circle)
;; is to go for 300 - 700 (400km buffer around the inner circle, smaller inner circle)
;; Sticking with 500 1000 for the paper, treating the 300-700 in the supplementary materials
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;




;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; Calculate which cities and outbreaks occurred unexpectedly near sites of tree-ring records, so we later on
;; can look at the climate preceding these outbreaks.
;;
(def reservoirs-500-1000
  (let [raw-with-BD (apply merge
                      (for [[indicator info] (select-keys climate-info (filter i-europe (keys climate-info)))]
                        (hash-map indicator (return-potential-outbreaks (:center info) 500 1000))))]
        ;; with Black Death removed
        (zipmap (keys raw-with-BD)
                  (map (fn [coll]
                     (let [raw (zipmap (keys coll) (map #(set/difference % #{1347 1348 1349 1350 1351 1352 1353}) (vals coll)))]
                       (select-keys raw (keys (remove (fn [[k v]] (empty? v)) raw))))) (vals raw-with-BD)))))

(def reservoirs-500-1000-pruned
    (zipmap (keys reservoirs-500-1000)
        (let [events-splitup (map #(mapcat (fn [[c ys]] (map (fn [y] {c #{y}}) ys)) %) (vals reservoirs-500-1000))
              known-reintros #{;; AB Granville 1825 - on the Quarantine Bill
                               {"Marseille" #{1786}}
                               ;; CA Devaux 2012 - Small oversights.."
                               {"Marseille" #{1720}} {"Brignoles" #{1720}} {"Avignon" #{1720}} {"Orange" #{1720}} {"Apt" #{1720}} {"Tarascon" #{1720}} {"Toulon" #{1720}}
                               ;; F de Arrieta 1694 - Raguaglio historico del contaggio.."
                               {"Bari" #{1691}} {"Monopoli" #{1691}}
                               ;; AB Granville 1819 - A letter on the Plague and Contagion"
                               {"Noia" #{1815}}
                             }
              events-filtered (map (fn [coll] (apply merge-with set/union (set/difference (set coll) known-reintros))) events-splitup)]
      events-filtered)))

(def reservoirs-300-700
  (let [raw-with-BD (apply merge
                      (for [[indicator info] (select-keys climate-info (filter i-europe (keys climate-info)))]
                        (hash-map indicator (return-potential-outbreaks (:center info) 300 700))))]
        ;; with Black Death removed
        (zipmap (keys raw-with-BD)
                  (map (fn [coll]
                     (let [raw (zipmap (keys coll) (map #(set/difference % #{1347 1348 1349 1350 1351 1352 1353}) (vals coll)))]
                       (select-keys raw (keys (remove (fn [[k v]] (empty? v)) raw))))) (vals raw-with-BD)))))

(def reservoirs-300-700-pruned
    (zipmap (keys reservoirs-300-700)
        (let [events-splitup (map #(mapcat (fn [[c ys]] (map (fn [y] {c #{y}}) ys)) %) (vals reservoirs-300-700))
              known-reintros #{;; AB Granville 1825 - on the Quarantine Bill
                               {"Marseille" #{1786}}
                               ;; CA Devaux 2012 - Small oversights.."
                               {"Marseille" #{1720}} {"Brignoles" #{1720}} {"Avignon" #{1720}} {"Orange" #{1720}} {"Apt" #{1720}} {"Tarascon" #{1720}} {"Toulon" #{1720}}
                               ;; F de Arrieta 1694 - Raguaglio historico del contaggio.."
                               {"Bari" #{1691}} {"Monopoli" #{1691}}
                               ;; AB Granville 1819 - A letter on the Plague and Contagion"
                               {"Noia" #{1815}}
                             }
              events-filtered (map (fn [coll] (apply merge-with set/union (set/difference (set coll) known-reintros))) events-splitup)]
      events-filtered)))

;; Some summary statistics of the output
;;
;; For 500km inner circle - 1000km outter circle
;;
;; number of cities, 19. 39 with 300/700
(count (apply merge-with set/union (vals reservoirs-500-1000)))

;; number of city-times=years, 24, and 59 with 300/700
(apply + (map count (vals (apply merge-with set/union (vals reservoirs-500-1000)))))

;; number of cities, 12, after pruning
(count (apply merge-with set/union (vals reservoirs-500-1000-pruned)))

;; number of city-times=years, 16, after pruning
(apply + (map count (vals (apply merge-with set/union (vals reservoirs-500-1000-pruned)))))

;; For 300km inner circle - 700km outter circle
;;
;; number of cities, 19. 39 with 300/700
(count (apply merge-with set/union (vals reservoirs-300-700)))

;; number of city-times=years, 24, and 59 with 300/700
(apply + (map count (vals (apply merge-with set/union (vals reservoirs-300-700)))))

;; number of cities, 12, after pruning, 34 with 300/700
(count (apply merge-with set/union (vals reservoirs-300-700-pruned)))

;; number of city-times=years, 16, after pruning, 51 with 300/700
(apply + (map count (vals (apply merge-with set/union (vals reservoirs-300-700-pruned)))))
;;
;;
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;





;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; FIGURE S2AB - POTENTIAL RESERVOIR OUTBREAKS in Europe
;;
;; Plotted for two different thresholds
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(prepare-kavraiskiy-europe)
(reset! zoom 10)
(def x-offset (atom -260))
(def y-offset (atom -280))
;; 1027 pixels = 8.7cm and 300DPI.
;; Doesn't fit in 8.7cm though. Just too small.
;; Haven't plotted it at the 2100px size, just half resolution. So scale up for final prints.
(q/defsketch europe-map :setup setup-europe :size [(/ 2100 2) (/ 1610 2)]
  :renderer :pdf
  :output-file (str "results/FIGURE-S2-europeanreservoirs-500-1000.pdf")
  :draw (fn []
          ;; Background
          (q/background 150 185 230)
          (q/stroke 250 240 220) (q/fill 250 240 220) (plot-shapes landshapes :c1 -40 :c2 140 :c3 -10 :c4 85)
          (q/stroke-weight 1)
          (q/stroke 0 0 0 0) (q/fill 150 185 230) (plot-shapes caspian-sea)
          (q/stroke 0 0 0 0) (q/fill 150 185 230) (plot-shapes lakes)
          (q/stroke-weight 1.5)
          ;; Plague
          (doseq [[x y n] (map #(conj (latlon-to-xy (city-to-latlon %)) %) (sort-by #(count (plague-db-rollupyears %)) (keys plague-db-rollupyears)))]
            (let [c (count (plague-db-rollupyears n))]
              (if false #_(harbors n) (do (q/stroke 0 0 0) (q/fill 0 0 0)) (do (q/stroke 200) (q/fill 220)))
              (q/ellipse x y (+ 3 (sqrt (/ (* 7 c) q/PI))) (+ 3 (sqrt (/ (* 7 c) q/PI))))))
          ;; Tree-rings, outter edge 1000km -example albania, inner edge 500km
          (q/stroke-weight 2)
          (q/stroke 50 200 50 200)
          (let [[cx cy] (:center (climate-info :z-score-wettemp-albania))]
            (doseq [[s1 s2] (map vector (range 0 360 0.1) (conj (vec (range 0.1 360 0.1)) 0))]
              (let [dir-1 (last (take-while #(< (globe-distance [cx cy] [(+ cx (* % (sin (q/radians s1)))) (+ cy (* % (cos (q/radians s1))))]) 500) (range 1 20 0.01)))
                    dir-2 (last (take-while #(< (globe-distance [cx cy] [(+ cx (* % (sin (q/radians s2)))) (+ cy (* % (cos (q/radians s2))))]) 500) (range 1 20 0.01)))
                    [x1 y1] [(+ cx (* dir-1 (sin (q/radians s1)))) (+ cy (* dir-1 (cos (q/radians s1))))]
                    [x2 y2] [(+ cx (* dir-2 (sin (q/radians s2)))) (+ cy (* dir-2 (cos (q/radians s2))))]]
                   (let [[px1 py1] (latlon-to-xy [x1 y1]) [px2 py2] (latlon-to-xy [x2 y2])]
                         (q/line px1 py1 px2 py2)))))
          (q/stroke 200 50 50 100)
          (let [[cx cy] (:center (climate-info :z-score-wettemp-albania))]
            (doseq [[s1 s2] (map vector (range 0 360 0.1) (conj (vec (range 0.1 360 0.1)) 0))]
              (let [dir-1 (last (take-while #(< (globe-distance [cx cy] [(+ cx (* % (sin (q/radians s1)))) (+ cy (* % (cos (q/radians s1))))]) 1000) (range 1 20 0.01)))  ;; when the last range is too small, the picture doesn't make sense (loses its projection-correct shape, turns into a circle)
                    dir-2 (last (take-while #(< (globe-distance [cx cy] [(+ cx (* % (sin (q/radians s2)))) (+ cy (* % (cos (q/radians s2))))]) 1000) (range 1 20 0.01)))
                    [x1 y1] [(+ cx (* dir-1 (sin (q/radians s1)))) (+ cy (* dir-1 (cos (q/radians s1))))]
                    [x2 y2] [(+ cx (* dir-2 (sin (q/radians s2)))) (+ cy (* dir-2 (cos (q/radians s2))))]]
                   (let [[px1 py1] (latlon-to-xy [x1 y1]) [px2 py2] (latlon-to-xy [x2 y2])]
                         (q/line px1 py1 px2 py2)))))
          (q/stroke-weight 1)
          (q/stroke 100 200 100 250) (q/fill 100 200 100 150)
          (doseq [[x y] (map latlon-to-xy (map :center (vals (select-keys climate-info (filter i-europe (keys climate-info))))))]
            (q/ellipse x y 30 30))
          (q/stroke 0) (q/fill 0)
          ;; Locations and years of plague with potential European reservoirs.
          ;;
          (doseq [[x y c] (map #(conj (latlon-to-xy (city-to-latlon %)) %) (reverse (sort-by #(count (plague-db-rollupyears %)) (keys (apply merge-with set/union (vals reservoirs-500-1000))))))]
            (let [[x2 y2] (case c
                           "Limburg an der Lahn" [5 -2]
                           "Frankfurt am Main" [5 3]
                           "Ston" [-65 -3]
                           "Baden bei Wien" [4 10]
                           "Wien" [5 0]
                           "Dubrovnik" [10 -3]
                           "Bari" [-15 -6]
                           "Marseille" [-55 17]
                           "Würzburg" [7 8]
                           "Samborz" [1 -3]
                           "Alaise" [5 1]
                           "Kortrijk" [-2 -5]
                           "Tournai" [-35 15]
                           [5 5])]
              (q/text-size 14)
              (q/stroke 120)
              ;; Color blue these circles, which are all near Marseille, and are part of two maritime reintroductions of plague. Else color the names black, and the circles orange
              (if (#{"Marseille" "Tarascon" "Avignon" "Brignoles" "Orange" "Toulon" "Apt"} c) (q/fill 80 100 220) (q/fill 255 165 0))
              (let [[x y] (latlon-to-xy (city-to-latlon c))]
                (q/ellipse x y (+ 3 (sqrt (/ (* 7 (count (plague-db-rollupyears c))) q/PI))) (+ 3 (sqrt (/ (* 7 (count (plague-db-rollupyears c))) q/PI)))))
              (if (#{"Marseille" "Tarascon" "Avignon" "Brignoles" "Orange" "Toulon" "Apt"} c) (q/fill 80 100 220) (q/fill 0))
              ;; Conventional formatting of names
              (if (not (#{"Marseille" "Tarascon" "Avignon" "Brignoles" "Orange" "Toulon" "Apt"} c))
                (q/text (apply str (concat [c ", "] (interpose ", " (sort (set ((apply merge-with set/union (vals reservoirs-500-1000)) c)))))) (+ x x2) (+ y y2)))
              ;; If it is Marseille, format the names a bit different.
              (when (= c "Marseille")
                (q/fill 80 100 220)
                (q/text (str "Marseille, " (apply str (interpose ", " (drop 1 (sort (set ((apply merge-with set/union (vals reservoirs-500-1000)) c))))))) (+ x x2) (+ y y2 16))
                (q/fill 80 100 220)
                (q/text "Marseille & surroundings, 1720" (+ x x2) (+ y y2 3)))
         (q/exit)
            ))))

(q/defsketch europe-map :setup setup-europe :size [(/ 2100 2) (/ 1610 2)]
  :renderer :pdf
  :output-file (str "results/FIGURE-S2-europeanreservoirs-300-700.pdf")
  :draw (fn []
          (q/background 150 185 230) ;; Ocean
          (q/stroke 250 240 220) (q/fill 250 240 220) (plot-shapes landshapes :c1 -40 :c2 140 :c3 -10 :c4 85)
          (q/stroke-weight 1)
          (q/stroke 0 0 0 0) (q/fill 150 185 230) (plot-shapes caspian-sea)
          (q/stroke 0 0 0 0) (q/fill 150 185 230) (plot-shapes lakes)
          (q/stroke-weight 1.5)
          ;; Plague
          (doseq [[x y n] (map #(conj (latlon-to-xy (city-to-latlon %)) %) (sort-by #(count (plague-db-rollupyears %)) (keys plague-db-rollupyears)))]
            (let [c (count (plague-db-rollupyears n))]
              (if false #_(harbors n) (do (q/stroke 0 0 0) (q/fill 0 0 0)) (do (q/stroke 200) (q/fill 220)))
              (q/ellipse x y (+ 3 (sqrt (/ (* 7 c) q/PI))) (+ 3 (sqrt (/ (* 7 c) q/PI))))))
          ;; Tree-rings, outter edge 1000km -example albania, inner edge 500km
          (q/stroke-weight 2)
          (q/stroke 50 200 50 200)
          (let [[cx cy] (:center (climate-info :z-score-wettemp-albania))]
            (doseq [[s1 s2] (map vector (range 0 360 0.1) (conj (vec (range 0.1 360 0.1)) 0))]
              (let [dir-1 (last (take-while #(< (globe-distance [cx cy] [(+ cx (* % (sin (q/radians s1)))) (+ cy (* % (cos (q/radians s1))))]) 300) (range 1 20 0.01)))
                    dir-2 (last (take-while #(< (globe-distance [cx cy] [(+ cx (* % (sin (q/radians s2)))) (+ cy (* % (cos (q/radians s2))))]) 300) (range 1 20 0.01)))
                    [x1 y1] [(+ cx (* dir-1 (sin (q/radians s1)))) (+ cy (* dir-1 (cos (q/radians s1))))]
                    [x2 y2] [(+ cx (* dir-2 (sin (q/radians s2)))) (+ cy (* dir-2 (cos (q/radians s2))))]]
                   (let [[px1 py1] (latlon-to-xy [x1 y1]) [px2 py2] (latlon-to-xy [x2 y2])]
                         (q/line px1 py1 px2 py2)))))
          (q/stroke 200 50 50 100)
          (let [[cx cy] (:center (climate-info :z-score-wettemp-albania))]
            (doseq [[s1 s2] (map vector (range 0 360 0.1) (conj (vec (range 0.1 360 0.1)) 0))]
              (let [dir-1 (last (take-while #(< (globe-distance [cx cy] [(+ cx (* % (sin (q/radians s1)))) (+ cy (* % (cos (q/radians s1))))]) 700) (range 1 20 0.01)))  ;; when the last range is too small, the picture doesn't make sense (loses its projection-correct shape, turns into a circle)
                    dir-2 (last (take-while #(< (globe-distance [cx cy] [(+ cx (* % (sin (q/radians s2)))) (+ cy (* % (cos (q/radians s2))))]) 700) (range 1 20 0.01)))
                    [x1 y1] [(+ cx (* dir-1 (sin (q/radians s1)))) (+ cy (* dir-1 (cos (q/radians s1))))]
                    [x2 y2] [(+ cx (* dir-2 (sin (q/radians s2)))) (+ cy (* dir-2 (cos (q/radians s2))))]]
                   (let [[px1 py1] (latlon-to-xy [x1 y1]) [px2 py2] (latlon-to-xy [x2 y2])]
                         (q/line px1 py1 px2 py2)))))
          (q/stroke-weight 1)
          (q/stroke 100 200 100 250) (q/fill 100 200 100 150)
          (doseq [[x y] (map latlon-to-xy (map :center (vals (select-keys climate-info (filter i-europe (keys climate-info))))))]
            (q/ellipse x y 30 30))
          (q/stroke 0) (q/fill 0)
          (doseq [[x y c] (map #(conj (latlon-to-xy (city-to-latlon %)) %) (reverse (sort-by #(count (plague-db-rollupyears %)) (keys (apply merge-with set/union (vals reservoirs-300-700))))))]
            (let [[x2 y2] (case c
                           "Barcelona" [7 15]
                           "Gerona" [0 15]
                           "Bordeaux" [9 5]
                           "Zaragoza" [-40 16]
                           "Villefranche-de-Rouergue" [-200 17]
                           "Rodez" [9 5]
                           "Basel" [10 5]
                           "Salzburg" [5 12]
                           "Montpellier" [-165 7]
                           "Hamburg" [9 5]
                           "Bremen" [9 5]
                           "Toulouse" [-73 -5]
                           "Erfurt" [5 8]
                           "Halle (Saale)" [5 -5]
                           "Leipzig" [5 -2]
                           "Bari" [-15 -6]
                           "Aix-en-Provence" [7 -2]
                           "Monopoli" [-60 11]
                           "Marseille" [-55 17]
                           "Würzburg" [7 8]
                           "Samborz" [1 -3]
                           "Limburg an der Lahn" [-170 -7]
                           "Alaise" [5 1]
                           "Frankfurt am Main" [-175 7]
                           "Manacor" [5 -4]
                           "Nordhausen" [-93 5]
                           "Alberstedt" [-83 -5]
                           "Eisleben Lutherstadt" [0 12]
                           "Castres" [-14 12]
                           "Šibenik" [4 -1]
                           "Ston" [-65 -3]
                           "Baden bei Wien" [7 7]
                           "Bourg-en-Bresse" [-80 -12]
                           "Eichstätt" [-85 -5]
                           "Stuttgart" [-40 15]
                           "Regensburg" [5 0]
                           "Poznan" [7 5]
                           "Wien" [7 0]
                           "Paris" [-7 -9]
                           "Nice" [3 12]
                           "Saint-Malo" [-8 13]
                           "Saint-Denis" [5 5]
                           "Noja" [-10 14]
                           "Venezia" [5 1]
                           "Palma" [-60 15]
                           "Orange" [-60 -10]
                           "Genova" [7 5]
                           "Kortrijk" [-2 -5]
                           "Dubrovnik" [7 -8]
                           "Kotor" [-10 13]
                           "Edirne" [-52 13]
                           "Tournai" [-35 15]
                           "Gardelegen" [-90 3]
                           "Constantine" [-50 15]
                           "Bone" [-40 -8]
                           "Tunis" [0 -8]
                           "Zarzis" [5 0]
                           [5 5])]
              (q/text-size 14)
              (q/stroke 120)
              (if (#{"Noja" "Split" "Siacco" "Genova" #_"Dubrovnik" "Aix-en-Provence" "Marseille" "Monopoli" "Tarascon" "Brignoles" "Orange" "Toulon" "Avignon" "Apt"} c) (q/fill 80 100 220) (q/fill 255 165 0))
              (let [[x y] (latlon-to-xy (city-to-latlon c))]
                (q/ellipse x y (+ 3 (sqrt (/ (* 7 (count (plague-db-rollupyears c))) q/PI))) (+ 3 (sqrt (/ (* 7 (count (plague-db-rollupyears c))) q/PI)))))
              ;; Black Death, smugglers at Noja, Smugglers at Monopoli/Bari, Marseille
              (if (#{"Noja" "Split" "Siacco" "Genova" #_"Dubrovnik" "Aix-en-Provence" "Marseille" "Monopoli"} c) (q/fill 80 100 220) (q/fill 0))
              (if (not (#{"Tarascon" "Brignoles" "Orange" "Toulon" "Avignon" "Dubrovnik" "Apt" "Marseille"} c))
                (q/text (apply str (concat [c ", "] (interpose ", " (sort (set ((apply merge-with set/union (vals reservoirs-300-700)) c)))))) (+ x x2) (+ y y2)))
              (when (= c "Marseille")
                (q/fill 0)
                (q/text (str "Marseille, " (apply str (interpose ", " (take 1 (sort (set ((apply merge-with set/union (vals reservoirs-300-700)) c))))))) (+ x x2) (+ y y2))
                (q/fill 80 100 220)
                (q/text (str "Marseille, " (apply str (interpose ", " (drop 1 (sort (set ((apply merge-with set/union (vals reservoirs-300-700)) c))))))) (+ x x2 -70) (+ y y2 13))
                (q/text "; surroundings, 1720" (+ x x2 76) (+ y y2 13)))
              (when (= c "Dubrovnik")
                (q/fill 0)
                (q/text "Dubrovnik, " (+ x x2) (+ y y2))
                (q/text (apply str (interpose ", " (take 7 (sort (set ((apply merge-with set/union (vals reservoirs-300-700)) c)))))) (+ x x2 85) (+ y y2))
                (q/text (apply str (interpose ", " (drop 7 (sort (set ((apply merge-with set/union (vals reservoirs-300-700)) c)))))) (+ x x2 85) (+ y y2 13)))
         (q/exit)
            ))))
;;
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;






;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; SUPPLEMENTARY FIGURE S3 - CLIMATE PRECEDING NEAR-PROXY OUTBREAKS,
;; excluding maritime imports
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(doseq [[indicator outbreaklist] (zipmap (keys reservoirs-500-1000-pruned) (map #(vec (sort (apply set/union %))) (map vals (vals reservoirs-500-1000-pruned))))]
  (let [outbreaks (map #(+ % 4) (sort outbreaklist))
        climate (apply merge (map #(hash-map (:year %) (indicator %)) climate-db-raw))
        plot (xy-plot (range -30 4) (repeat 0) :legend true  :x-label "Number of years prior to outbreak" :y-label "Tree-ring growth conditions" :title (:name (climate-info indicator)))]
      (set-stroke plot :dataset (last-series plot) :dash 4)
      (set-stroke-color plot java.awt.Color/gray :dataset (last-series plot))
      (doseq [year (sort outbreaks)]
        ;; plot year-line in gray
        (add-lines plot (range -30 4) (map climate (range (- year 34) (inc year))) :series-label (str (- year 4)))
        (set-stroke plot :dataset (last-series plot) :dash 2)
        (set-stroke-color plot #_java.awt.Color/black (color-list (count (take-while #(not= year %) outbreaks))) :dataset (last-series plot)))
      (remove-from-legend plot 0)
      (.setStandardTickUnits (.getDomainAxis (.getPlot plot)) (org.jfree.chart.axis.NumberAxis/createIntegerTickUnits))
      (add-lines plot [0 0] [-5 5] :series-label "")
      (set-stroke-color plot java.awt.Color/gray :dataset (last-series plot))
      (add-pointer plot 0.1 -3.6 :text "Year of outbreak" :color java.awt.Color/black :angle :nw)
      (theme-plot plot)
      (set-y-range plot -3.5 3)
      (set-x-range plot -20 3)
      (save-pdf plot (str "results/FIGURE-S3/FIGURE-S3-europe-" (name indicator) "-500-1000-climatepreceding.pdf") :width (* 2 648) :height 400)
      (view plot :width (* 2 648) :height 400)))

(doseq [[indicator outbreaklist] (zipmap (keys reservoirs-500-1000-pruned) (map #(vec (sort (apply set/union %))) (map vals (vals reservoirs-300-700-pruned))))]
  (let [outbreaks (map #(+ % 4) (sort outbreaklist))
        climate (apply merge (map #(hash-map (:year %) (indicator %)) climate-db-raw))
        plot (xy-plot (range -30 4) (repeat 0) :legend true  :x-label "Number of years prior to outbreak" :y-label "Tree-ring growth conditions" :title (:name (climate-info indicator)))]
      (set-stroke plot :dataset (last-series plot) :dash 4)
      (set-stroke-color plot java.awt.Color/gray :dataset (last-series plot))
      (doseq [year (sort outbreaks)]
        ;; plot year-line in gray
        (add-lines plot (range -30 4) (map climate (range (- year 34) (inc year))) :series-label (str (- year 4)))
        (set-stroke plot :dataset (last-series plot) :dash 2)
        (set-stroke-color plot #_java.awt.Color/black (color-list (count (take-while #(not= year %) outbreaks))) :dataset (last-series plot)))
      (remove-from-legend plot 0)
      (.setStandardTickUnits (.getDomainAxis (.getPlot plot)) (org.jfree.chart.axis.NumberAxis/createIntegerTickUnits))
      (add-lines plot [0 0] [-5 5] :series-label "")
      (set-stroke-color plot java.awt.Color/gray :dataset (last-series plot))
      (add-pointer plot 0.1 -3.6 :text "Year of outbreak" :color java.awt.Color/black :angle :nw)
      (theme-plot plot)
      (set-y-range plot -3.5 3)
      (set-x-range plot -20 3)
      (save-pdf plot (str "results/FIGURE-S3/FIGURE-S3-europe-" (name indicator) "-300-700-climatepreceding.pdf") :width (* 2 648) :height 400)
      (view plot :width (* 2 648) :height 400)))
;;
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;






;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;
;; Figure S7, random distribution investigations.
;;
;;
;;
;; Preparing for randomization test
;; We looked at several ways to do the randomization (including the simplest). We settled
;; for something where we preserve the clustering of outbreak years on a scale up to half
;; a century, meaning that the gap distribution for gaps < 50 years between outbreaks is
;; not more frequent in the random sample for any gapsize 1-50, than in the observed sample.
;;
;; Just shuffling the gaps (which would have been another option) was getting us into trouble,
;; as the large gaps of 200 years or so in the reintroduction years forces a particular distribution
;; in the random samples (many of the short gaps at the outter edges of 1330-1850, long gaps in
;; the middle), that is not particularly wanted.
;;
;; Just randomly pulling 16 years from 1330-1850 had as disadvantage that frequenty the randomized
;; sample had a gap distribution more dense then the observed, leading to a lower number of possible
;; matches between unique climate fluctuations and reintroduction years, thus overestimating the
;; significance of our result by 1-5%.
;;
;; Slightly ahead of the logic of the narrative in the paper, we also look at the potential
;; reintroduction years here from Asia to Europe, which we only calculate later in the
;; in this program.
;;
;;;;;;;;;;;;;;;;;;;;
(defn gen-simple-random
  "Generates a set of years between 1330 and 1851 of equal size of coll (up to size 1850-1330)"
  [coll] (sort (take (count coll) (shuffle (range 1330 1851)))))

(defn calc-gapsizes [sorted-coll]
  (into (sorted-map) (for [gapsize (range 1 101)] (vector gapsize (count (filter #(<= (* -1 gapsize) (apply - %)) (partition 2 1 sorted-coll)))))))

(defn good-gapsizes [random-gapsizes tresh observed-gapsizes]
  "test whether the years in the randomly drawn reintroduction years are not too close together (which would unfairly favor the observed correlation).
  Doing this only for gaps up to 170 in length"
  (not (some (fn [[gap obs-freq]] (and (<= gap tresh) (> (get random-gapsizes gap 0) obs-freq))) observed-gapsizes)))

;; Example: the random gapsizes are closer together than the observed gapsizes (the latter coll), so it isn't good.
;; Being equally close together is fine.
;;(good-gapsizes (calc-gapsizes [1 2 4 5]) 100 (calc-gapsizes [1 2 4 6]))
;;(good-gapsizes (calc-gapsizes [1]) 100 (calc-gapsizes [2]))

;; Some repetition here in code, but basically functions that
;; generate 50,000 randomized series of years for the permutation test
(comment
  ;; Disabled for now, and reading in the data earlier calculated.
  ;; If you want to recreate the data, remove the comment.
(let [reservoirs (into (hash-map) (filter (fn [[k v]] (seq v)) reservoirs-500-1000-pruned))
      randomized (for [[i outbreaks] reservoirs]
                    (let [observed-series (sort (apply set/union (vals outbreaks)))
                          observed-gapsizes (calc-gapsizes observed-series)]
                      (loop [bucket []]
                        (let [randomized-series (gen-simple-random observed-series)]
                          (if (< (count bucket) 50000)
                            (recur (if (good-gapsizes (calc-gapsizes randomized-series) 50 observed-gapsizes) (conj bucket randomized-series) bucket))
                            (hash-map i (hash-map :series observed-series :randomized bucket)))))))]
  (def randomized-europe-proxies-500-1000 (apply merge randomized)))
(spit "results/randomized-europe-proxies-500-1000.txt" randomized-europe-proxies-500-1000)

(let [reservoirs (into (hash-map) (filter (fn [[k v]] (seq v)) reservoirs-300-700-pruned))
      randomized (for [[i outbreaks] reservoirs]
                    (let [observed-series (sort (apply set/union (vals outbreaks)))
                          observed-gapsizes (calc-gapsizes observed-series)]
                      (loop [bucket []]
                        (let [randomized-series (gen-simple-random observed-series)]
                          (if (< (count bucket) 50000)
                            (recur (if (good-gapsizes (calc-gapsizes randomized-series) 50 observed-gapsizes) (conj bucket randomized-series) bucket))
                            (hash-map i (hash-map :series observed-series :randomized bucket)))))))]
  (def randomized-europe-proxies-300-700 (apply merge randomized)))
(spit "results/randomized-europe-proxies-300-700.txt" randomized-europe-proxies-300-700)

(let [randomized (let [observed-series [1346, 1408, 1409, 1689, 1693, 1719, 1730, 1737, 1757, 1760, 1762, 1780, 1783, 1828, 1830, 1837]
                       observed-gapsizes (calc-gapsizes observed-series)]
                      (loop [bucket []]
                        (let [randomized-series (gen-simple-random observed-series)]
                          (if (< (count bucket) 50000)
                            (recur (if (good-gapsizes (calc-gapsizes randomized-series) 50 observed-gapsizes) (conj bucket randomized-series) bucket))
                            (hash-map :series observed-series :randomized bucket)))))]
  (def randomized-asia2europe randomized))
(spit "results/randomized-asia2europe.txt" randomized-asia2europe)

(let [randomized (let [observed-series [1346, 1408, 1409, 1689, 1693, 1719, 1730, 1737, 1757, 1760, 1762, 1780, 1783, 1828, 1830, 1837]
                       observed-gapsizes (calc-gapsizes observed-series)]
                      (loop [bucket []]
                        (let [randomized-series (gen-simple-random observed-series)]
                          (if (< (count bucket) 50000)
                            (recur (if (good-gapsizes (calc-gapsizes randomized-series) 100 observed-gapsizes) (conj bucket randomized-series) bucket))
                            (hash-map :series observed-series :randomized bucket)))))]
  (def randomized-asia2europe-100 randomized))
(spit "results/randomized-asia2europe-100.txt" randomized-asia2europe-100)
)

(def randomized-europe-proxies-500-1000 (read-string (slurp "results/randomized-europe-proxies-500-1000.txt")))
(def randomized-europe-proxies-300-700 (read-string (slurp "results/randomized-europe-proxies-300-700.txt")))
(def randomized-asia2europe (read-string (slurp "results/randomized-asia2europe.txt")))
(def randomized-asia2europe-100 (read-string (slurp "results/randomized-asia2europe-100.txt")))
;;
;; Look at the distribution of years in the samples.
;; When going to a 100 years as aggregation threshold, the initial and last years 30-ish of the timeframe (1330-1360 and 1820-1850)
;; are picked more frequently than the years in the middle, just to confirm to the rules by which
;; the randomized series need to be spaced. By reducing it to 50 years,
(let [data-50 (into (sorted-map) (frequencies (flatten (:randomized randomized-asia2europe))))
      data-100 (into (sorted-map) (frequencies (flatten (:randomized randomized-asia2europe-100))))
      data-random (into (sorted-map) (frequencies (repeatedly (* 50000 (count [1346, 1408, 1409, 1689, 1693, 1719, 1730, 1737, 1757, 1760, 1762, 1780, 1783, 1828, 1830, 1837])) #(+ (rand-int (- 1851 1330)) 1330))))
      plot (doto (xy-plot (keys data-random) (vals data-random) :series-label "No clustering conserved" :legend true :x-label "Year" :y-label "Frequency")
            (add-lines (keys data-100) (vals data-100) :series-label "Clustering conserved up to 100 years")
            (add-lines (keys data-50) (vals data-50) :series-label "Clustering conserved up to 50 years")
            set-theme-bw
            (set-y-range 1000 2250))]
    (.setStandardTickUnits (.getDomainAxis (.getPlot plot)) (org.jfree.chart.axis.NumberAxis/createIntegerTickUnits))
    (save-pdf plot "results/FIGURE-S7A-randomdistribution-east2west.pdf" :width 800)
    (view plot :width 800))

(comment
  ;; Idem as the randomization plot for the east-to-west plague reintroductions, but then for the various intervals between
  ;; potential local reservoir-related outbreaks of Europe.
(doseq [[i d] randomized-europe-proxies-500-1000]
   (let [data (into (sorted-map) (frequencies (flatten (:randomized d))))]
      (view (xy-plot (keys data) (vals data)))))

(doseq [[i d] randomized-europe-proxies-300-700]
   (let [data (into (sorted-map) (frequencies (flatten (:randomized d))))]
      (view (xy-plot (keys data) (vals data)))))
)
;;
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;







;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; Calculate significance of found potential reservoir reintroductions
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;; Helper and plotting functions
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defn peak-height
  "Only calculates the height of the the highest peak in the distribution of intervals between climate and reintroductions"
  [drop-years intro-years from till tolerance]
  (let [tolerance-range (range (* -1 tolerance) (inc tolerance))
        answer (apply merge-with concat (flatten (for [drop drop-years intro intro-years :when (<= from (- drop intro) till)]
                                                          (map #(hash-map (+ % (- drop intro)) [drop]) tolerance-range))))]
    (if (seq answer)
        (apply max (map #(count (set %)) (vals answer))) ;; The set makes it so that the same climat event cannot be counted twice.
      0)))

(defn delay-dist
  "calculates how the intervals are distributed between climate events and plague introductions. Uses the unique climate events, not the total number of events,
  to avoid years like 1408 and 1409 referring to the same climate event as counting as two reintroductions."
  [drop-years intro-years from till tolerance]
  (let [tolerance-range (range (* -1 tolerance) (inc tolerance))
        answer (apply merge-with concat (flatten (for [drop drop-years intro intro-years :when (<= from (- drop intro) till)]
                                                          (map #(hash-map (+ % (- drop intro)) [drop]) tolerance-range))))]
    (if (seq answer)
        (hash-map :maxpeak (apply max (map #(count (set %)) (vals answer))) ;; The set makes it so that the same climate event cannot be counted twice.
                  :delay-dist-withtol (frequencies (mapcat (fn [[k v]] (repeat (count v) k)) answer))
                  :raw answer)
      (hash-map :maxpeak 0 :delay-dist-withtol {} :raw {}))))

(defn draw-heatmap-permutationresults [i outbreaklist observed-perm random-perm filename]
  (let [plot (scatter-plot [] [] :x-label "Cumulative above-average period" :y-label "Amplitude of continuous decline" :title (:name (climate-info i)))
        stroke (java.awt.BasicStroke. 2 java.awt.BasicStroke/CAP_ROUND java.awt.BasicStroke/JOIN_ROUND 1.0 (float-array 1.0 1) 0.0)
        numrepeats 50000
        testing-range (map float (range 1 5.2 0.2))
        p-val 0.95
        un-freq (fn [coll] (if (seq coll) (sort (mapcat (fn [[k v]] (repeat v k)) coll)) (repeat numrepeats 0)))
        max-delay (fn [coll] (reverse (sort (map #(* -1 (first %)) (last (partition-by second (sort-by second coll)))))))]
  (doseq [t1 testing-range t2 testing-range]
    (let [aspect 1
          diff2 (if (< 0 (- (:maxpeak (observed-perm [i t1 t2])) (first (drop (int (* numrepeats p-val)) (un-freq (random-perm [i t1 t2])))))) ;; absolute peak height
                   (:maxpeak (observed-perm [i t1 t2]))
                 0)
          numintros (if (vals (:delay-dist-withtol (observed-perm [i t1 t2]))) (apply max (vals (:delay-dist-withtol (observed-perm [i t1 t2])))) 0)
          pval (float (/ (count (drop-while #(< % (:maxpeak (observed-perm [i t1 t2]))) (un-freq (random-perm [i t1 t2])))) numrepeats))
          size 1.25
          obj (new java.awt.geom.Rectangle2D$Double (- t1 (* size 0.07)) (- t2 (* size 0.07)) (* aspect size (+ 0.07 0.07)) (* size (+ 0.07 0.07)))]
      (when (< pval 0.05)
        (.addAnnotation (.getPlot plot) (new org.jfree.chart.annotations.XYShapeAnnotation obj stroke
                                             (new java.awt.Color (int (* 5000 (max 0 (- 0.05 pval)))) 0 0)
                                             (new java.awt.Color (int (* 5000 (max 0 (- 0.05 pval)))) 0 0)))
        (let [anno (new org.jfree.chart.annotations.XYTextAnnotation (apply str (interpose "," (max-delay (:delay-dist-withtol (observed-perm [i t1 t2]))))) t1 (+ t2 0.06))
              anno2 (new org.jfree.chart.annotations.XYTextAnnotation (str pval) t1 t2)
              anno3 (new org.jfree.chart.annotations.XYTextAnnotation (str diff2 ":" numintros "/" (count outbreaklist)) t1 (- t2 0.06))]
          (.setPaint anno java.awt.Color/white)
          (if (> (count (max-delay (:delay-dist-withtol (observed-perm [i t1 t2])))) 1) (.setFont anno (new Font "Arial" Font/BOLD 8)))
          (.setPaint anno2 java.awt.Color/white)
          (.setFont anno2 (new Font "Arial" Font/BOLD 8))
          (.setPaint anno3 java.awt.Color/white)
          (.addAnnotation (.getPlot plot) anno)
          (.addAnnotation (.getPlot plot) anno2)
          (.setFont anno3 (new Font "Arial" Font/BOLD 9))
          (.addAnnotation (.getPlot plot) anno3)))))
  (.setRangeGridlinesVisible (.getPlot plot) false)
  (.setDomainGridlinesVisible (.getPlot plot) false)
  (.setLabelPaint (.getDomainAxis (.getPlot plot)) java.awt.Color/gray)
  (.setTickLabelPaint (.getDomainAxis (.getPlot plot)) java.awt.Color/gray)
  (.setTickMarkPaint (.getDomainAxis (.getPlot plot)) java.awt.Color/gray)
  (.setAxisLinePaint (.getDomainAxis (.getPlot plot)) java.awt.Color/gray)
  (.setLabelPaint (.getRangeAxis (.getPlot plot)) java.awt.Color/gray)
  (.setTickLabelPaint (.getRangeAxis (.getPlot plot)) java.awt.Color/gray)
  (.setTickMarkPaint (.getRangeAxis (.getPlot plot)) java.awt.Color/gray)
  (.setAxisLinePaint (.getRangeAxis (.getPlot plot)) java.awt.Color/gray)
  (set-x-range plot 0.85 5.15)
  (set-y-range plot 0.85 5.15)
  (save-pdf plot (str "results/FIGURE-S4/" filename "-" (name i) ".pdf") :width (* 880) :height (* 810))
  (view plot :height 810 :width 880)))

(defn draw-heatmap-tiles [i outbreaklist observed-perm random-perm filename]
  (let [plot (scatter-plot [] [] :x-label "" :y-label "")
        stroke (java.awt.BasicStroke. 2 java.awt.BasicStroke/CAP_ROUND java.awt.BasicStroke/JOIN_ROUND 1.0 (float-array 1.0 1) 0.0)
        numrepeats 50000
        testing-range (map float (range 1 5.2 0.2))
        p-val 0.95
        un-freq (fn [coll] (if (seq coll) (sort (mapcat (fn [[k v]] (repeat v k)) coll)) (repeat numrepeats 0)))
        max-delay (fn [coll] (reverse (sort (map #(* -1 (first %)) (last (partition-by second (sort-by second coll)))))))]
  (doseq [t1 testing-range t2 testing-range]
    (let [aspect 1
          pval (float (/ (count (drop-while #(< % (:maxpeak (observed-perm [i t1 t2]))) (un-freq (random-perm [i t1 t2])))) numrepeats))
          size 1.1
          obj (new java.awt.geom.Rectangle2D$Double (- t1 (* size 0.07)) (- t2 (* size 0.07)) (* aspect size (+ 0.07 0.07)) (* size (+ 0.07 0.07)))]
      (when (< pval 0.05)
        (.addAnnotation (.getPlot plot) (new org.jfree.chart.annotations.XYShapeAnnotation obj stroke
                                             (new java.awt.Color (int (* 5000 (max 0 (- 0.05 pval)))) 0 0)
                                             (new java.awt.Color (int (* 5000 (max 0 (- 0.05 pval)))) 0 0))))))
  (add-subtitle plot (:name (climate-info i)))
  (.setRangeGridlinesVisible (.getPlot plot) false)
  (.setDomainGridlinesVisible (.getPlot plot) false)
  (.setLabelPaint (.getDomainAxis (.getPlot plot)) java.awt.Color/gray)
  (.setTickLabelPaint (.getDomainAxis (.getPlot plot)) java.awt.Color/gray)
  (.setTickMarkPaint (.getDomainAxis (.getPlot plot)) java.awt.Color/gray)
  (.setAxisLinePaint (.getDomainAxis (.getPlot plot)) java.awt.Color/gray)
  (.setLabelPaint (.getRangeAxis (.getPlot plot)) java.awt.Color/gray)
  (.setTickLabelPaint (.getRangeAxis (.getPlot plot)) java.awt.Color/gray)
  (.setTickMarkPaint (.getRangeAxis (.getPlot plot)) java.awt.Color/gray)
  (.setAxisLinePaint (.getRangeAxis (.getPlot plot)) java.awt.Color/gray)
  (set-x-range plot 0.85 5.15)
  (set-y-range plot 0.85 5.15)
  (.setStandardTickUnits (.getDomainAxis (.getPlot plot)) (org.jfree.chart.axis.NumberAxis/createIntegerTickUnits))
  (.setStandardTickUnits (.getRangeAxis (.getPlot plot)) (org.jfree.chart.axis.NumberAxis/createIntegerTickUnits))
  (save-pdf plot (str "results/FIGURE-3A-S5/FIGURE-3A-" filename "-" (name i) ".pdf") :width (* 200) :height (* 200))
  (view plot :width (* 200) :height (* 200))))

;; The actual function that extracts climate fluctuations from the tree-ring record, as well as translates these
;; to discrete event years. We explore (on the editors' request) several options.
;;
;; Option A, our default.
;;
;; Discrete year is the first year of the continuous decline.
;; Also determines years involved in cumulative above-average period, and the
;; total dropsize
;;;;;;;;;;;;;;;;;;;;;;
(defn find-big-drops [indicator positivethreshold mindropsize clim-db]
  "Calculates from when to when climate fluctuations occur, based on a period of above positivethreshold (usually 0),
  followed by a decline. When the mindropsize is set to 0, the decline must be continuous (our default).
  Might break in unsuspecting ways if you change positivethreshold and mindropsize to odd values though. Haven't tested
  all possible use-cases.

  As default, we use :year-cd as the discrete year which defined the climate fluctuation, the dropsize is then :drop-cd,
  and the above-average period is then sumabove-uptilcd. Some of the other possibilities are tested in the supp materials
  to test the robustness of our method for translating treerings to plague-relevant climate fluctuations to climate events.
  "
  (let [climate (map #(hash-map :year (ffirst %) :score (second (first %)) :deltanext (- (second (second %)) (second (first %)))) (partition 2 1 (remove #(= (second %) nil) (sort-by first (map #(vector (:year %) (get % indicator)) clim-db)))))
        rawdata (filter #(and (> (:score (first %)) positivethreshold) (< (apply min (map :deltanext %)) mindropsize)) (partition-by #(or (> (:score %) positivethreshold) (< (:deltanext %) mindropsize)) climate))]
    (map (fn [entry]
           (let [extended-entry (concat (map vector (map :year entry) (map :score entry)) (list (vector (inc (:year (last entry))) (+ (:score (last entry)) (:deltanext (last entry))))))
                 peak (apply max (map second extended-entry))
                 start-peak (first (last (filter #(= (second %) peak) extended-entry))) ;; last-found highest peak.
                 bottom (apply min (map second extended-entry))
                 start-continuousdecline (let [yearprior-continuousdecline (:year (first (drop-while #(< (:deltanext %) mindropsize) (reverse entry))))]
                                           (if yearprior-continuousdecline (inc yearprior-continuousdecline) (:year (first entry))))
                 peak-continuousdecline (second (first (filter #(= (first %) start-continuousdecline) extended-entry)))]
             (hash-map :year-cd start-continuousdecline
                       :year-peak start-peak
                       :start (:year (first entry))
                       :end (inc (:year (last entry)))
                       :drop-cd (- peak-continuousdecline bottom)
                       :drop (- peak bottom)
                       :peak peak
                       :bottom bottom
                       :seq (vec extended-entry)
                       :sumabove-uptilpeak (apply + (map #(max 0 %) (minus (map second (filter #(<= (first %) start-peak) extended-entry)) positivethreshold)))
                       :sumabove-uptilcd (apply + (map #(max 0 %) (minus (map second (filter #(<= (first %) start-continuousdecline) extended-entry)) positivethreshold))))))
         rawdata)))


(comment
;;
;; Run the permutation test for Europe 500-1000
;;
(let [indicators i-europe
      [c-eventyear c-dropsize c-sumabove] [:year-cd :drop-cd :sumabove-uptilcd]
      from -20 till 0 tolerance 1
      all-climate-drops (zipmap indicators (map #(find-big-drops % 0 -0 climate-db-raw) indicators))
      observed-perm (apply merge (for [i indicators t-plateau (map float (range 1 5.2 0.2)) t-drop (map float (range 1 5.2 0.2))]
                      (hash-map [i t-plateau t-drop]
                        (let [clim-drops (map c-eventyear (filter #(and (>= (c-sumabove %) t-plateau) (>= (c-dropsize %) t-drop)) (all-climate-drops i)))]
                          (delay-dist clim-drops (:series (randomized-europe-proxies-500-1000 i)) from till tolerance)))))]
    (spit "results/permutation-test-observed-europe-reservoirs-500-1000.txt" observed-perm))

(let [random-years randomized-europe-proxies-500-1000
      [c-eventyear c-dropsize c-sumabove] [:year-cd :drop-cd :sumabove-uptilcd]
      indicators i-europe
      from -20 till 0 tolerance 1
      all-climate-drops (zipmap indicators (map #(find-big-drops % 0 -0 climate-db-raw) indicators))
      random-perm (apply merge
                     (flatten (pmap (fn [i]
                         (doall (for [t-plateau (map float (range 1 5.2 0.2)) t-drop (map float (range 1 5.2 0.2))]
                            (hash-map [i t-plateau t-drop]
                              (let [clim-drops (map c-eventyear (filter #(and (>= (c-sumabove %) t-plateau) (>= (c-dropsize %) t-drop)) (all-climate-drops i)))]
                                 (frequencies (map #(peak-height clim-drops % from till tolerance) (:randomized (random-years i)))))))))
                             indicators)))]
    (spit "results/permutation-test-randomexpectation-europe-reservoirs-500-1000.txt" random-perm))
;;
;; Run the permutation test for Europe 300-700
;;
(let [indicators i-europe
      [c-eventyear c-dropsize c-sumabove] [:year-cd :drop-cd :sumabove-uptilcd]
      from -20 till 0 tolerance 1
      all-climate-drops (zipmap indicators (map #(find-big-drops % 0 -0 climate-db-raw) indicators))
      observed-perm (apply merge (for [i indicators t-plateau (map float (range 1 5.2 0.2)) t-drop (map float (range 1 5.2 0.2))]
                      (hash-map [i t-plateau t-drop]
                        (let [clim-drops (map c-eventyear (filter #(and (>= (c-sumabove %) t-plateau) (>= (c-dropsize %) t-drop)) (all-climate-drops i)))]
                          (delay-dist clim-drops (:series (randomized-europe-proxies-300-700 i)) from till tolerance)))))]
  (spit "results/permutation-test-observed-europe-reservoirs-300-700.txt" observed-perm))

(let [random-years randomized-europe-proxies-300-700
      [c-eventyear c-dropsize c-sumabove] [:year-cd :drop-cd :sumabove-uptilcd]
      indicators i-europe
      from -20 till 0 tolerance 1
      all-climate-drops (zipmap indicators (map #(find-big-drops % 0 -0 climate-db-raw) indicators))
      random-perm (apply merge
                   (flatten (pmap (fn [i]
                       (doall (for [t-plateau (map float (range 1 5.2 0.2)) t-drop (map float (range 1 5.2 0.2))]
                          (hash-map [i t-plateau t-drop]
                              (let [clim-drops (map c-eventyear (filter #(and (>= (c-sumabove %) t-plateau) (>= (c-dropsize %) t-drop)) (all-climate-drops i)))]
                                 (frequencies (map #(peak-height clim-drops % from till tolerance) (:randomized (random-years i)))))))))
                       indicators)))]
  (spit "results/permutation-test-randomexpectation-europe-reservoirs-300-700.txt" random-perm))
)

;; Just remember that floats become doubles when you read them back in, hence we turn them back into floats.
(let [r (read-string (slurp "results/permutation-test-observed-europe-reservoirs-300-700.txt"))]
  (def observed-perm-europe-reservoirs-300-700 (zipmap (map #(vec (concat [(first %)] (map float (rest %)))) (keys r)) (vals r))))
(let [r (read-string (slurp "results/permutation-test-randomexpectation-europe-reservoirs-300-700.txt"))]
  (def random-perm-europe-reservoirs-300-700 (zipmap (map #(vec (concat [(first %)] (map float (rest %)))) (keys r)) (vals r))))
(let [r (read-string (slurp "results/permutation-test-observed-europe-reservoirs-500-1000.txt"))]
  (def observed-perm-europe-reservoirs-500-1000 (zipmap (map #(vec (concat [(first %)] (map float (rest %)))) (keys r)) (vals r))))
(let [r (read-string (slurp "results/permutation-test-randomexpectation-europe-reservoirs-500-1000.txt"))]
  (def random-perm-europe-reservoirs-500-1000 (zipmap (map #(vec (concat [(first %)] (map float (rest %)))) (keys r)) (vals r))))
;;
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


;;;;;;;;;;;;;;;;;
;;
;; Figure S3, detailed results of the permutation test, for reintroductions from European reservoirs
;;
;; They'll end up in the directory for S4, but just move them to S3.
(doseq [[i outbreaklist] (zipmap (keys randomized-europe-proxies-500-1000) (map :series (vals randomized-europe-proxies-500-1000)))]
  (draw-heatmap-permutationresults i outbreaklist observed-perm-europe-reservoirs-500-1000  random-perm-europe-reservoirs-500-1000 "european-proxies-500-1000"))
(doseq [[i outbreaklist] (zipmap (keys randomized-europe-proxies-500-1000) (map :series (vals randomized-europe-proxies-300-700)))]
  (draw-heatmap-permutationresults i outbreaklist observed-perm-europe-reservoirs-300-700  random-perm-europe-reservoirs-300-700 "european-proxies-300-700"))





;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; NEXT STEP, HARBORS.
;;
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(defn return-potential-outbreaks-city [city nearby-cities nearby-harbors]
  (let [new-outbreaks (set (mingap-years (sort (plague-db-rollupyears city)) 2)) ;; no plague outbreaks in that city in the previous 2 years.
        buffer-outbreaks-extended (set (mapcat #(range % (+ % 2)) (set (mapcat (fn [c] (plague-db-rollupyears c)) nearby-cities)))) ;; outbreaks in the buffer in the previous years as a new-outbreak disqualify the source idea. Excluding nearby harbors.
        harbour-outbreaks (set (mapcat #(range % (+ % 2)) (set (mapcat (fn [c] (plague-db-rollupyears c)) nearby-harbors))))] ;; no outbreaks within NNN km nearby in the 2 years prior?
    (hash-map city (set/difference new-outbreaks buffer-outbreaks-extended harbour-outbreaks))))

;; Harbour outbreaks, not within 1000km of each other. Using with 1500km works as well, if not better.
(let [sentinels (set/intersection (set (keys plague-db-rollupyears)) harbors)
      answer (apply merge
                    (map (fn [city]
                           (let [nearby-landcities (set/difference (local-cities (city-to-latlon city) 500) sentinels)
                                 nearby-harbors (set/difference (set/intersection (local-cities (city-to-latlon city) 1000) sentinels) #{city})]
                           (return-potential-outbreaks-city city nearby-landcities nearby-harbors)))
                         sentinels))
      raw answer]
  (def v-raw raw))


;;;;;;;;;;;;;;;;;;;;
;;
;; FIGURE 1, PAPER
;;
;;
(prepare-kavraiskiy-europe2)
(def y-offset (atom -100))
(q/defsketch europe-map :setup setup-europe :size [874 600]
  :renderer :pdf
  :output-file (str "results/FIGURE-1-possiblereintros-harbors.pdf")
  :draw (fn []
          (q/background 150 185 230) ;; Ocean
          (q/stroke-weight 1.5)
          (q/stroke 110 140 200) (q/fill 0 0 0 0) (plot-shapes searoutes)
          (q/stroke-weight 1)
          (q/stroke 250 240 220) (q/fill 250 240 220) (plot-shapes landshapes :c1 -40 :c2 140 :c3 -10 :c4 85)
          (q/stroke 0 0 0 0) (q/fill 150 185 230) (plot-shapes caspian-sea)
          (q/stroke 150 185 230) (q/fill 150 185 230) (plot-shapes lakes)
          (q/stroke 250 200 70) (q/fill 250 200 70) (plot-shapes sylvaticplague)
          (q/stroke-weight 2)
          (q/stroke-weight 1.5)
          (q/stroke 130 130 70) (q/fill 0 0 0 0) (plot-shapes silkroad)
          (q/stroke 130 130 70) (q/fill 0 0 0 0) (plot-shapes tearoad)
          ;; Plague
          (doseq [[x y n] (map #(conj (latlon-to-xy (city-to-latlon %)) %) (sort-by #(count (plague-db-rollupyears %)) (remove harbors (keys plague-db-rollupyears))))]
            (let [c (count (plague-db-rollupyears n))]
              (q/stroke 220)
              (q/fill 220)
              (q/ellipse x y (+ 1 (sqrt (/ (* 7 c) q/PI))) (+ 1 (sqrt (/ (* 7 c) q/PI))))))
          (doseq [[x y n] (map #(conj (latlon-to-xy (city-to-latlon %)) %) (sort-by #(count (plague-db-rollupyears %)) (filter harbors (keys plague-db-rollupyears))))]
            (let [c (count (plague-db-rollupyears n))]
              (q/stroke 0 0 0)
              (q/fill 220)
              (q/ellipse x y (+ 1 (sqrt (/ (* 7 c) q/PI))) (+ 1 (sqrt (/ (* 7 c) q/PI))))))
          ;; Plague
          (q/stroke 0) (q/fill 0)
          (doseq [[x y c] (map #(conj (latlon-to-xy (city-to-latlon %)) %) (keys v-raw))]
            (let [[x2 y2] (case c
                           "Bordeaux" [9 4]
                           "Montpellier" [7 -2]
                           "Valencia" [-50 10]
                           "Marseille" [7 8]
                           "Barcelona" [8 4]
                           "Constantinople" [8 3]
                           "Oran" [-45 15]
                           "London" [-30 -12]
                           "Southampton" [-50 10]
                           "Sandwich" [4 -1]
                           "Brugge" [7 8]
                           "Hamburg" [7 9]
                           "Alger" [-7 19]
                           "Palermo" [-50 3]
                           "Tunis" [-7 16]
                           "Izmir" [10 4]
                           "Bone" [-15 -5]
                           "Almería" [7 0]
                           "Málaga" [6 9]
                           "Cádiz" [-7 13]
                           "Lübeck" [7 5]
                           "Rostock" [7 1]
                           [7 4])]
              ;(q/ellipse x y 3 3)
              (q/text-size 11)
              (q/fill 0)
              (if (#{"Alger"} c)
                (do
                  (q/text (apply str (concat [c] (if (seq (v-raw c)) (concat [", "] (interpose ", " (sort (set/intersection (set (range 1300 1900)) (take 2 (sort (set (v-raw c))))))))))) (+ x x2) (+ y y2))
                  (q/text (apply str (interpose ", " (sort (set/intersection (set (range 1300 1900)) (take 3 (drop 2 (sort (set (v-raw c))))))))) (+ x x2) (+ y y2 12))
                  (q/text (apply str (interpose ", " (sort (set/intersection (set (range 1300 1900)) (take 3 (drop 5 (sort (set (v-raw c))))))))) (+ x x2) (+ y y2 24))
                  (q/text (apply str (interpose ", " (sort (set/intersection (set (range 1300 1900))         (drop 7 (sort (set (v-raw c)))))))) (+ x x2) (+ y y2 36))
                  )
                (if (#{"Constantinople" "Alexandrie" "Izmir"} c)
                  (do
                    (q/text (apply str (concat [c] (if (seq (v-raw c)) (concat [", "] (interpose ", " (sort (set/intersection (set (range 1300 1900)) (take 4 (sort (set (v-raw c))))))))))) (+ x x2) (+ y y2))
                    (q/text (apply str (interpose ", " (sort (set/intersection (set (range 1300 1900)) (take 6 (drop 4 (sort (set (v-raw c))))))))) (+ x x2) (+ y y2 12))
                    (q/text (apply str (interpose ", " (sort (set/intersection (set (range 1300 1900)) (drop 10 (sort (set (v-raw c)))))))) (+ x x2) (+ y y2 24))
                  )
                  (do
                    (if (seq (v-raw c)) (q/fill 0) (q/fill 120))
                    (q/text (apply str (concat [c] (if (seq (v-raw c)) (concat [", "] (interpose ", " (sort (set/intersection (set (range 1300 1900)) (sort (set (v-raw c)))))))))) (+ x x2) (+ y y2)))))))
          (q/exit)
            ))

;; Summary statistics Figure 1
;;
;; harbors in dataset
(count (set/intersection (set (keys plague-db-rollupyears)) harbors))

;; number of unexpected outbreaks
(count (flatten (map vec (vals v-raw))))

;; unique years with unexpected outbreaks
(count (set (flatten (map vec (vals v-raw)))))

;; between years?
(apply min (set (flatten (map vec (vals v-raw)))))
(apply max (set (flatten (map vec (vals v-raw)))))

;; harbors with unexpected outbreaks
(count (filter (fn [[k v]] (seq v)) v-raw))


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; Determine the years where we have a potential reintroduction from Asia
;; using data from southern russia and the caucasus.
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


;; If unexpected plague in the east and west, and unexpected plague in europe a bit later.
(def east-to-west
    (into (sorted-set)
      (for [east (mingap-years (into (sorted-set) (mapcat #(mapcat (fn [c] (range (first c) (inc (last c)))) %) (vals (select-keys asian-plague [:southernrussia :caucasus])))) 2)
            west (mingap-years (sort (apply concat (vals v-raw))) 0) :when (<= 0 (- west east) 5)]
                            west)))

(count east-to-west)
east-to-west



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; Permutation test for reintroductions of plague from Asia
;;
(comment
  ;; A, version where size of decline is equal to event year definition:
  ;; [c-eventyear c-dropsize c-sumabove] [:year-cd :drop-cd :sumabove-uptilcd]
  ;;
  ;; B, version where event year definition is the peak (last one if multiple equal peaks), sumabove is up to that peak, and drop is from that peak:
  ;; [c-eventyear c-dropsize c-sumabove] [:year-peak :drop :sumabove-uptilpeak]
  ;;
  ;; C, like A, but with the event year defined as the initial year of the climate fluctuation:
  ;; [c-eventyear c-dropsize c-sumabove] [:start :drop-cd :sumabove-uptilcd]
  ;;
  ;; D, like A, but with the event year defined as the last year of the climate fluctuation:
  ;; [c-eventyear c-dropsize c-sumabove] [:end :drop-cd :sumabove-uptilcd]

(doseq [[variant keywords] [["A" [:year-cd :drop-cd :sumabove-uptilcd]]
                            ["B" [:year-peak :drop    :sumabove-uptilpeak]]
                            ["C" [:start     :drop-cd :sumabove-uptilcd]]
                            ["D" [:end       :drop-cd :sumabove-uptilcd]]]]
  (let [intro-years east-to-west
        indicators i-asia
        [c-eventyear c-dropsize c-sumabove] keywords
        from -20 till 0 tolerance 1
        all-climate-drops (zipmap indicators (map #(find-big-drops % 0 -0 climate-db-raw) indicators))
        observed-perm (apply merge (for [i indicators t-plateau (map float (range 1 5.2 0.2)) t-drop (map float (range 1 5.2 0.2))]
                        (hash-map [i t-plateau t-drop]
                          (let [clim-drops (map c-eventyear (filter #(and (>= (c-sumabove %) t-plateau) (>= (c-dropsize %) t-drop)) (all-climate-drops i)))]
                            (delay-dist clim-drops intro-years from till tolerance)))))]
    (spit (str "results/permutation-test-observed-asia2europe-" variant ".txt") observed-perm))
  (let [random-years (:randomized randomized-asia2europe)
        intro-years east-to-west
        indicators i-asia
        [c-eventyear c-dropsize c-sumabove] keywords
        from -20 till 0 tolerance 1
        all-climate-drops (zipmap indicators (map #(find-big-drops % 0 -0 climate-db-raw) indicators))
        random-perm (apply merge
                       (flatten (pmap (fn [i]
                           (doall (for [t-plateau (map float (range 1 5.2 0.2)) t-drop (map float (range 1 5.2 0.2))]
                              (hash-map [i t-plateau t-drop]
                                (let [clim-drops (map c-eventyear (filter #(and (>= (c-sumabove %) t-plateau) (>= (c-dropsize %) t-drop)) (all-climate-drops i)))]
                                   (frequencies (map #(peak-height clim-drops % from till tolerance) random-years)))))))
                                indicators)))]
    (spit (str "results/permutation-test-randomexpectation-asia2europe-" variant ".txt") random-perm)))

;; Repeat of D, but with a higher degree of temporal tolerance
(doseq [[variant keywords] [["D" [:end       :drop-cd :sumabove-uptilcd]]]]
  (let [intro-years east-to-west
        indicators i-asia
        [c-eventyear c-dropsize c-sumabove] keywords
        from -20 till 0 tolerance 2
        all-climate-drops (zipmap indicators (map #(find-big-drops % 0 -0 climate-db-raw) indicators))
        observed-perm (apply merge (for [i indicators t-plateau (map float (range 1 5.2 0.2)) t-drop (map float (range 1 5.2 0.2))]
                        (hash-map [i t-plateau t-drop]
                          (let [clim-drops (map c-eventyear (filter #(and (>= (c-sumabove %) t-plateau) (>= (c-dropsize %) t-drop)) (all-climate-drops i)))]
                            (delay-dist clim-drops intro-years from till tolerance)))))]
    (spit (str "results/permutation-test-observed-asia2europe-" variant "-tolerance2.txt") observed-perm))
  (let [random-years (:randomized randomized-asia2europe)
        intro-years east-to-west
        indicators i-asia
        [c-eventyear c-dropsize c-sumabove] keywords
        from -20 till 0 tolerance 2
        all-climate-drops (zipmap indicators (map #(find-big-drops % 0 -0 climate-db-raw) indicators))
        random-perm (apply merge
                       (flatten (pmap (fn [i]
                           (doall (for [t-plateau (map float (range 1 5.2 0.2)) t-drop (map float (range 1 5.2 0.2))]
                              (hash-map [i t-plateau t-drop]
                                (let [clim-drops (map c-eventyear (filter #(and (>= (c-sumabove %) t-plateau) (>= (c-dropsize %) t-drop)) (all-climate-drops i)))]
                                   (frequencies (map #(peak-height clim-drops % from till tolerance) random-years)))))))
                                indicators)))]
    (spit (str "results/permutation-test-randomexpectation-asia2europe-" variant "-tolerance2.txt") random-perm)))

)

;;;;;;;;;;;;;;;;;
;;
;; Figure S4, detailed results of the permutation test, for reintroductions from European reservoirs
;;
(let [r (read-string (slurp "results/permutation-test-observed-asia2europe-A.txt"))]
  (def observed-perm-asia2europe (zipmap (map #(vec (concat [(first %)] (map float (rest %)))) (keys r)) (vals r))))
(let [r (read-string (slurp "results/permutation-test-randomexpectation-asia2europe-A.txt"))]
  (def random-perm-asia2europe (zipmap (map #(vec (concat [(first %)] (map float (rest %)))) (keys r)) (vals r))))
(doseq [[i outbreaklist] (zipmap i-asia (repeat east-to-west))]
  (draw-heatmap-permutationresults i outbreaklist observed-perm-asia2europe  random-perm-asia2europe "asian-reintroductions"))
;;
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;





;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; FIGURE 2 METHODS + EXAMPLE
;;
;; Color coding the climate events has been done manually in the manuscript.
;;
(let [climate-db climate-db-raw
      bigdrop-colors (atom [])
      [c-eventyear c-dropsize c-sumabove] [:year-cd :drop-cd :sumabove-uptilcd]
      indicator :wettemp-holocene
      extend-list 7
      outbreaklist  [[1346] [1408 1409] [1757 1760 1762]]
      climate (into (sorted-map) (apply merge (remove #(nil? (first (vals %))) (map #(hash-map (:year %) (indicator %)) climate-db))))
      observed-drops (filter #(and (>= (c-sumabove %) 2.2) (>= (c-dropsize %) 1.4)) (find-big-drops indicator 0 0 climate-db))]
  (doseq [years outbreaklist]
    (let [plot (doto (xy-plot (range (- (first years) 22) (+ (first years) extend-list 2)) (map climate (range (- (first years) 22) (+ (first years) extend-list 2))) :y-label "Climate fluctations" :x-label "year")
                     (add-points years (repeat -2))
                     (set-stroke-color java.awt.Color/black :dataset 1)
                     set-theme-bw)
          drops (filter #(<= (- (first years) 20) (c-eventyear %) (+ (first years) extend-list)) observed-drops)]
      (doseq [[y d] (zipmap (map c-eventyear drops) (map :seq drops))]
        (when (not= y 1347) ;; remove the climate event that falls after 1346, in this example.
          (add-points plot [y] [(second (first (filter (fn [[k v]] (= y k)) d)))])
          (set-stroke-color plot java.awt.Color/black :dataset (last-series plot))
          (.setSeriesShape (.getRenderer (.getPlot plot) (last-series plot)) 0 (new java.awt.geom.Ellipse2D$Double -5.0, -5.0, 10.0, 10.0))
          (add-lines plot (map first d) (map second d))
          (set-stroke-color plot java.awt.Color/black :dataset (last-series plot))
          (set-stroke plot :width 2 :dataset (last-series plot))))
      (.setStandardTickUnits (.getDomainAxis (.getPlot plot)) (org.jfree.chart.axis.NumberAxis/createIntegerTickUnits))
      (.setStandardTickUnits (.getRangeAxis (.getPlot plot)) (org.jfree.chart.axis.NumberAxis/createIntegerTickUnits))
      (set-x-range plot (- (first years) 20 0.5) (+ (first years) extend-list))
      (save-pdf plot (str "results/FIGURE-2ABC-" (first years) ".pdf") :width 800 :height 200)
      (view plot :width 800 :height 200))))

(let [climate-db climate-db-raw
      outbreaklist [1346 1408 1409 1757 1760 1762]
      indicator :wettemp-holocene
      [c-eventyear c-dropsize c-sumabove] [:year-cd :drop-cd :sumabove-uptilcd]
      climate (into (sorted-map) (apply merge (remove #(nil? (first (vals %))) (map #(hash-map (:year %) (indicator %)) climate-db))))
      observed-drops (filter #(and (>= (c-sumabove %) 2.2) (>= (c-dropsize %) 1.4)) (find-big-drops indicator 0 0 climate-db))
      answer (into (sorted-map) (apply merge-with concat (mapcat (fn [year] (map #(hash-map (- year (c-eventyear %)) [(c-eventyear %)]) (filter #(and (<= (- year 20) (:end %)) (< (:start %) (inc year))) observed-drops))) outbreaklist)))
      data (dataset [:v :d :g]  (filter #((set (range 0 21)) (first %)) (reverse (sort-by first (concat (mapcat (fn [[k v]] (map #(vector (* 1 k) 1 %) v)) answer) (for [r (range 0 21)] [r 0 0]))))))
      plot (with-data data (stacked-bar-chart :v :d :group-by :g :legend true :x-label "Delay in years" :y-label "Frequency"))
      data2 (map #(vector (second %) (count (set (remove nil? (mapcat (fn [n] (if (<= 0 n 20) (get answer n nil) nil)) %))))) (partition 3 1 (range -1 22)))
      plot2 (xy-plot)]
  (set-y-range plot 0 10)
  (.setAxisLineVisible (.getDomainAxis (.getPlot plot)) true)
  (.setTickLabelPaint (.getDomainAxis (.getPlot plot)) java.awt.Color/gray)
  (.setTickMarkPaint (.getDomainAxis (.getPlot plot)) java.awt.Color/gray)
  (.setAxisLinePaint (.getDomainAxis (.getPlot plot)) java.awt.Color/gray)
  (.setAxisLineVisible (.getRangeAxis (.getPlot plot)) true)
  (.setTickLabelPaint (.getRangeAxis (.getPlot plot)) java.awt.Color/gray)
  (.setTickMarkPaint (.getRangeAxis (.getPlot plot)) java.awt.Color/gray)
  (.setAxisLinePaint (.getRangeAxis (.getPlot plot)) java.awt.Color/gray)
  (clear-background plot)
  (remove-from-legend plot 0)
  (let [sorted-items (sort-by #(.getDescription %) (map #(.get (.getLegendItems (.getPlot plot)) %) (range 0 6)))
        newlegend (new org.jfree.chart.LegendItemCollection)]
        (dorun (for [r sorted-items] (.add newlegend r)))
        (.setFixedLegendItems (.getPlot plot) newlegend))
  (save-pdf plot (str "results/FIGURE-2D-2.pdf") :width 800 :height 320)
  (view plot :width 800 :height 320)
  ;; Plot 2.
  ;; No way to merge these plots in jfreechart.
  (add-lines plot2 (range -20 1) (reverse (map second data2)))
  (set-stroke-color plot2 java.awt.Color/black :dataset (last-series plot2))
  (set-x-range plot2 -21.1 1.5)
  (set-y-range plot2 0 10)
  (clear-background plot2)
  (.setRangeGridlinesVisible (.getPlot plot2) false)
  (.setDomainGridlinesVisible (.getPlot plot2) false)
  (.setAxisLineVisible (.getRangeAxis (.getPlot plot2)) true)
  (.setTickLabelPaint (.getRangeAxis (.getPlot plot2)) java.awt.Color/gray)
  (.setTickMarkPaint (.getRangeAxis (.getPlot plot2)) java.awt.Color/gray)
  (.setAxisLinePaint (.getRangeAxis (.getPlot plot2)) java.awt.Color/gray)
  (save-pdf plot2 (str "results/FIGURE-2D-1.pdf") :width 800 :height 295)
  (view plot2 :width 800 :height 295))
;;
;;
;;;;;;;;;;;;;;;;;;




;;;;;;;;;;;;;;;;
;;
;; Smaller version of the results of S4.
;;
;; Figure S5, and Figure 3A, overview of the permutation test
;;
(doseq [append ["A" "B" "C" "D"]]
  (let [r (read-string (slurp (str "results/permutation-test-observed-asia2europe-" append ".txt")))
        observed-perm-asia2europe (zipmap (map #(vec (concat [(first %)] (map float (rest %)))) (keys r)) (vals r))
        r2 (read-string (slurp (str "results/permutation-test-randomexpectation-asia2europe-" append ".txt")))
        random-perm-asia2europe (zipmap (map #(vec (concat [(first %)] (map float (rest %)))) (keys r2)) (vals r2))]
    (doseq [[i outbreaklist] (zipmap i-asia (repeat east-to-west))]
        (draw-heatmap-tiles i outbreaklist observed-perm-asia2europe  random-perm-asia2europe append))))
;; Can't find a way to automatically tile pdfs and save them in vector format. Used the autoaligning grid in inkscape.


;; For the alternative algorithms in Fig S7, using the final year of the climate fluctuation as the climate event year
;; destroys the significan correlation
;;
;; Read in the extra tolerance version (+- 2years)
;;
(let [r (read-string (slurp "results/permutation-test-observed-asia2europe-D-tolerance2.txt"))]
  (def observed-perm-asia2europe (zipmap (map #(vec (concat [(first %)] (map float (rest %)))) (keys r)) (vals r))))
(let [r (read-string (slurp "results/permutation-test-randomexpectation-asia2europe-D-tolerance2.txt"))]
  (def random-perm-asia2europe (zipmap (map #(vec (concat [(first %)] (map float (rest %)))) (keys r)) (vals r))))
(doseq [[i outbreaklist] (zipmap i-asia (repeat east-to-west))]
  (draw-heatmap-permutationresults i outbreaklist observed-perm-asia2europe  random-perm-asia2europe "asian-reintroductions-D-tol2"))

;;;;;;;;;;;;;;;;
;;
;; Fig. S6. How do the climate event years shift when we change the algorithms (see S5)
;;
;; So, is the variation between :year-cd and :end bigger than between :year-cd and :yeak-peak, or between :year-cd and :start?
;; Prettify tomorrow.
(let [data (filter #(and (>= (:drop-cd %) 1) (>= (:sumabove-uptilcd %) 1)) (find-big-drops :wettemp-holocene 0 0 climate-db-raw))
      d1 (into (sorted-map) (frequencies (map #(- (:end %) (:year-cd %)) data)))
      d2 (into (sorted-map) (frequencies (map #(- (:year-cd %) (:start %)) data)))
      d3 (into (sorted-map) (frequencies (map #(- (:year-cd %) (:year-peak %)) data)))
      plot (xy-plot (keys d1) (vals d1) :x-label "Difference in years" :y-label "Frequency" :legend true :series-label "end-year of climate fluctuation minus begin-year of decline")]
    (set-theme-bw plot)
    (set-stroke-color plot java.awt.Color/gray :dataset (last-series plot))
    (add-points plot (keys d1) (vals d1) :series-label "")
    (set-stroke-color plot java.awt.Color/gray :dataset (last-series plot))
    (add-points plot (keys d2) (vals d2) :series-label "")
    (set-stroke-color plot java.awt.Color/red :dataset (last-series plot))
    (add-lines plot (keys d2) (vals d2) :series-label "begin-year of decline minus start-year of climate fluctuation")
    (set-stroke-color plot java.awt.Color/red :dataset (last-series plot))
    (add-points plot (keys d3) (vals d3) :series-label "")
    (set-stroke-color plot java.awt.Color/blue :dataset (last-series plot))
    (add-lines plot(keys d3) (vals d3) :series-label "peak-year of climate fluctuation minus begin-year of decline")
    (set-stroke-color plot java.awt.Color/blue :dataset (last-series plot))
    (set-x-range plot -0.3 5.3)
    (.setStandardTickUnits (.getDomainAxis (.getPlot plot)) (org.jfree.chart.axis.NumberAxis/createIntegerTickUnits))
    (remove-from-legend plot 4)
    (remove-from-legend plot 2)
    (remove-from-legend plot 1)
    (save-pdf plot "results/FIGURE-S6A-difference-in-translation.pdf")
    (view plot))


;; Back to default.
(let [r (read-string (slurp "results/permutation-test-observed-asia2europe-A.txt"))]
  (def observed-perm-asia2europe (zipmap (map #(vec (concat [(first %)] (map float (rest %)))) (keys r)) (vals r))))
(let [r (read-string (slurp "results/permutation-test-randomexpectation-asia2europe-A.txt"))]
  (def random-perm-asia2europe (zipmap (map #(vec (concat [(first %)] (map float (rest %)))) (keys r)) (vals r))))



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; Figure 3B. Idem to figure 2D.
;;
;;
(let [climate-db climate-db-raw
      outbreaklist  east-to-west
      indicator :wettemp-holocene
      [c-eventyear c-dropsize c-sumabove] [:year-cd :drop-cd :sumabove-uptilcd]
      climate (into (sorted-map) (apply merge (remove #(nil? (first (vals %))) (map #(hash-map (:year %) (indicator %)) climate-db))))
      observed-drops (filter #(and (>= (c-sumabove %) 2.2) (>= (c-dropsize %) 1.4)) (find-big-drops indicator 0 0 climate-db))
      answer (into (sorted-map) (apply merge-with concat (mapcat (fn [year] (map #(hash-map (- year (c-eventyear %)) [(c-eventyear %)]) (filter #(and (<= (- year 20) (:end %)) (< (:start %) (inc year))) observed-drops))) outbreaklist)))
      data (dataset [:v :d :g]  (filter #((set (range 0 21)) (first %)) (reverse (sort-by first (concat (mapcat (fn [[k v]] (map #(vector (* 1 k) 1 %) v)) answer) (for [r (range 0 21)] [r 0 0]))))))
      plot (with-data data (stacked-bar-chart :v :d :group-by :g :legend true :x-label "Delay in years" :y-label "Test statistic score"))
      data2 (map #(vector (second %) (count (set (remove nil? (mapcat (fn [n] (if (<= 0 n 20) (get answer n nil) nil)) %))))) (partition 3 1 (range -1 22)))
      plot2 (xy-plot)
      pvalmap (let [d (into (sorted-map) (zipmap (keys (random-perm-asia2europe [:wettemp-holocene (float 2.2) (float 1.4)])) (map #(/ % 50000.0) (vals (random-perm-asia2europe [:wettemp-holocene (float 2.2) (float 1.4)])))))]
                  (into (sorted-map) (zipmap (reverse (keys d)) (reductions + (reverse (vals d))))))]
  (set-y-range plot 0 10)
  (.setAxisLineVisible (.getDomainAxis (.getPlot plot)) true)
  (.setTickLabelPaint (.getDomainAxis (.getPlot plot)) java.awt.Color/gray)
  (.setTickMarkPaint (.getDomainAxis (.getPlot plot)) java.awt.Color/gray)
  (.setAxisLinePaint (.getDomainAxis (.getPlot plot)) java.awt.Color/gray)
  (.setAxisLineVisible (.getRangeAxis (.getPlot plot)) true)
  (.setTickLabelPaint (.getRangeAxis (.getPlot plot)) java.awt.Color/gray)
  (.setTickMarkPaint (.getRangeAxis (.getPlot plot)) java.awt.Color/gray)
  (.setAxisLinePaint (.getRangeAxis (.getPlot plot)) java.awt.Color/gray)
  (clear-background plot)
  (remove-from-legend plot 0)
  (let [sorted-items (sort-by #(.getDescription %) (map #(.get (.getLegendItems (.getPlot plot)) %) (range 0 14)))
        newlegend (new org.jfree.chart.LegendItemCollection)]
        (dorun (for [r sorted-items] (.add newlegend r)))
        (.setFixedLegendItems (.getPlot plot) newlegend))
  (save-pdf plot (str "results/FIGURE-3B-2.pdf") :width 800 :height 320)
  (view plot :width 800 :height 320)
  ;; Plot 2.
  ;; No way to merge these plots in jfreechart.
  ;; P-values
  (add-lines plot2 (range -20 1) (repeat 9))
  (set-stroke-color plot2 java.awt.Color/gray :dataset (last-series plot2))
  (add-text plot2  -18.40 9.25 (format "p<%.5f" (pvalmap 9)))
  (add-lines plot2 (range -20 1) (repeat 8))
  (set-stroke-color plot2 java.awt.Color/gray :dataset (last-series plot2))
  (add-text plot2  -18.40 8.25 (format "p<%.5f" (pvalmap 8)))
  (add-lines plot2 (range -20 1) (repeat 7))
  (set-stroke-color plot2 java.awt.Color/gray :dataset (last-series plot2))
  (add-text plot2  -18.40 7.25 (format "p<%.5f" (pvalmap 7)))
  (add-lines plot2 (range -20 1) (reverse (map second data2)))
  (set-stroke-color plot2 java.awt.Color/black :dataset (last-series plot2))
  ;; Add another line with the expected number of peaks for p<0.01
  (set-x-range plot2 -21.1 1.5)
  (set-y-range plot2 0 10)
  (clear-background plot2)
  (.setRangeGridlinesVisible (.getPlot plot2) false)
  (.setDomainGridlinesVisible (.getPlot plot2) false)
  (.setAxisLineVisible (.getRangeAxis (.getPlot plot2)) true)
  (.setTickLabelPaint (.getRangeAxis (.getPlot plot2)) java.awt.Color/gray)
  (.setTickMarkPaint (.getRangeAxis (.getPlot plot2)) java.awt.Color/gray)
  (.setAxisLinePaint (.getRangeAxis (.getPlot plot2)) java.awt.Color/gray)
  (save-pdf plot2 (str "results/FIGURE-3B-1.pdf") :width 800 :height 295)
  (view plot2 :width 800 :height 295))
;;
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; Figure 3C.
;;
;; Using 14-15-16 years prior, and above/decline 2.2,1.4 as thresholds.
;;
(let [[c-eventyear c-dropsize c-sumabove] [:year-cd :drop-cd :sumabove-uptilcd]
      clim-events (map c-eventyear (filter #(and (>= (c-sumabove %) 2.2) (>= (c-dropsize %) 1.4)) (find-big-drops :wettemp-holocene 0 0 climate-db-raw)))
      clim-events-matched (set (flatten (vals (select-keys (:raw (observed-perm-asia2europe [:wettemp-holocene (float 2.2) (float 1.4)])) [-14 -15 -16]))))
      reintros-matched (set/intersection (set (flatten (map (fn [[k v]] (map #(- % k) v)) (select-keys (:raw (observed-perm-asia2europe [:wettemp-holocene (float 2.2) (float 1.4)])) [-14 -15 -16]))))
               east-to-west)
      reintros-mismatched (set/difference east-to-west reintros-matched)
      d (into (sorted-map) (vec (frequencies (map :year plague-db))))
      stroke (java.awt.BasicStroke. 2 java.awt.BasicStroke/CAP_SQUARE java.awt.BasicStroke/JOIN_MITER 1.0 (float-array 1.0 1) 0.0)
      plot (xy-plot (range 1685 1900) (div (map #(get d % 0) (range 1685 1900)) 16.66666) :x-label "Year" :y-label "Plague Incidence")]
  (add-lines plot (range 1200 1685) (div (map #(get d % 0) (range 1200 1685)) 100))
  (set-stroke-color plot java.awt.Color/gray :dataset 0)
  (set-stroke plot :dataset 0 :dash 2)
  (set-stroke-color plot java.awt.Color/black :dataset 1)
  ;; Plot all climate events. Those matched in red
  (doseq [p clim-events]
        (add-points plot [p] [-0.2])
        (.setSeriesShape (.getRenderer (.getPlot plot) (last-series plot)) 0 (new java.awt.geom.Rectangle2D$Double -3.0, -3.0, 6.0, 6.0))
        (set-stroke-color plot (if (clim-events-matched p) java.awt.Color/red java.awt.Color/gray)
                          :dataset (last-series plot))
        (add-lines plot [p (+ p 15)] [-0.2 0])
        (set-stroke-color plot (if (clim-events-matched p) java.awt.Color/red java.awt.Color/gray)
                          :dataset (last-series plot)))
  ;; Plot unmatched introduction events.
  ;(add-points plot reintros-mismatched (repeat 0))
  ;(.setSeriesShape (.getRenderer (.getPlot plot) (last-series plot)) 0 (new java.awt.geom.Ellipse2D$Double -3.0, -3.0, 6.0, 6.0))
  ;(set-stroke-color plot java.awt.Color/black :dataset (last-series plot))
  ;; Plot matched introduction events.
  (add-points plot reintros-matched (repeat 0))
  (.setSeriesShape (.getRenderer (.getPlot plot) (last-series plot)) 0 (new java.awt.geom.Ellipse2D$Double -3.0, -3.0, 6.0, 6.0))
  (set-stroke-color plot java.awt.Color/red :dataset (last-series plot))
  ;;; Theming.
  (theme-plot plot)
  (set-y-range plot -0.23 1.4)
  (set-x-range plot 1270 1851)
  (add-text plot 1783 0.85 "dashed line:")
  (add-text plot 1800 0.75 "incidence 6x magnified")
  (.setAxisLineVisible (.getRangeAxis (.getPlot plot)) false)
  (.setTickLabelsVisible (.getRangeAxis (.getPlot plot)) false)
  (.setTickMarksVisible (.getRangeAxis (.getPlot plot)) false)
  (.setStandardTickUnits (.getDomainAxis (.getPlot plot)) (org.jfree.chart.axis.NumberAxis/createIntegerTickUnits))
  (save-pdf plot (str "results/FIGURE-3C-climate-and-plagueincidence.pdf")  :width (* 1.0 800) :height 300)
  (view plot :width (* 1.0 800) :height 300))
;;
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;






;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; FIGURE 4.
;;

(prepare-carre-world)
(let [indicator :wettemp-holocene
      clim-events-matched (set (flatten (vals (select-keys (:raw (observed-perm-asia2europe [:wettemp-holocene (float 2.2) (float 1.4)])) [-14 -15 -16]))))]
  (q/defsketch europe :setup setup-europe :size [748 250]
    :renderer :pdf
    :output-file "results/FIGURE-4-PDSI.pdf"
    :draw (fn []
          (q/background 150 185 230) ;; Ocean
          (q/stroke 250 240 220) (q/fill 250 240 220) (plot-shapes landshapes)
          (q/stroke-weight 1.5)
          (q/stroke 110 140 200) (q/fill 0 0 0 0) (plot-shapes searoutes)
          (q/stroke-weight 1)
          (q/stroke 0 0 0 0) (q/fill 150 185 230) (plot-shapes caspian-sea)
          (q/stroke 0 0 0 0) (q/fill 150 185 230) (plot-shapes lakes)
          (q/stroke 250 200 70) (q/fill 250 200 70) (plot-shapes sylvaticplague)
          (q/stroke-weight 1.5)
          (q/stroke 130 130 70) (q/fill 0 0 0 0) (plot-shapes silkroad)
          (q/stroke 130 130 70) (q/fill 0 0 0 0) (plot-shapes tearoad)
          (q/stroke-weight 1)
          ;; difference in climate between onset of decline and 2 years later, shown for regions where this is negative.
          (let [summed-clim (merge-with - (apply merge-with + (map pdsi-asia (filter #(< 1300 % 1850) (map #(+ % 2) clim-events-matched))))
                                          (apply merge-with + (map pdsi-asia (filter #(< 1300 % 1850) (map #(+ % 0) clim-events-matched)))))]
            (doseq [[[lon lat] c] summed-clim]
              (let [[x y] (latlon-to-xy [lat lon])
                    normalizer (count clim-events-matched)]
                (q/stroke 0 0 0 0) (if (neg? c) (q/fill 230 50 0 (/ (* -250 0.25 c) normalizer)) (q/fill 0 0 0 0)) (q/rect (- x 5.5) (- y 5.5) 10 10))))
          ;; harbors responsible for plague
         (doseq [[x y c n] (map #(conj (latlon-to-xy (city-to-latlon (first %))) (first %) (count (second %))) v-raw)]
            ;; If a harbor doesn't have a contribution in terms of starting chains of maritime plague reintroductions
            (if (zero? n)
              (do (q/stroke-weight 1) (q/stroke 0) (q/fill 0) (q/ellipse x y 2 2))
              (do (q/stroke-weight 1.5) (q/stroke 0) (q/fill 150) (q/ellipse x y (sqrt (/ (* 100 n) q/PI)) (sqrt (/ (* 100 n) q/PI))))))
          ;; square around region that falls within PDSI
          (q/fill 0 0 0 0)
          (q/rect 400 24.5 500 300)
          (q/no-loop)
          (q/exit)
          )))
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;





;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;
;; Exporting data for autocorrelation / periodicity testing in R, using cpgram.
;;
(defn years-to-ts [coll]
  (vals (into (sorted-map) (merge
                (apply hash-map (interleave (range 1330 1851) (repeat 0)))
                (apply hash-map (interleave coll (repeat 1)))))))

(def e2w-ts (years-to-ts east-to-west))

;; Export to R
(spit "Interface-with-R/east2west_timeseries.csv"
      (apply str (concat (interpose ", " e2w-ts) ["\n"])))


